package com.winhc.bigdata.spark.etl

import java.util.Date

import com.winhc.bigdata.spark.udf.CompanyMapping
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, MaxComputer2Phoenix, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author yyn
 * @Date 2020/9/28
 * @Description 增量ods到ads的仅针对人员的同步
 */
case class PersonIncrSync(s: SparkSession,
                          project: String, //表所在工程名
                          tableName: String, //表名（不加前后辍）
                          idCardCol: String, //身份证字段
                          dupliCols: Seq[String], // 去重列
                          cidCol: String,//企业id字段名称，默认为"cid"
                          updateCol: String = "update_time" //ROW_NUMBER窗口函数的ORDER BY字段，默认（可以不传参数）为update_time
                                              ) extends LoggingUtils with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  def all(): Unit ={

    prepareFunctions(spark)
    val targetTab = s"$project.ads_${tableName}_person"
    val orgTab = s"$project.ods_${tableName}"

    val lastDs = getLastPartitionsOrElse(orgTab,"0")

    val columns: Seq[String] = getColumns(targetTab).filter(s => {
      !s.equals("ds") && !s.equals(s"new_$cidCol") && !s.equals("rowkey")
    })

    val cols_md5 = dupliCols.filter(!_.equals(s"new_$cidCol"))

    val personIdDs =getLastPartitionsOrElse("ads_person_idcard_cloze","0")
    val ods_company_tb = s"${project}.ods_$tableName" //增量ods表

    val runDs = getLastPartitionsOrElse(ods_company_tb,"0")
   sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${targetTab} PARTITION(ds=$lastDs)
         |SELECT  rowkey
         |        ,new_$cidCol
         |        ,$cidCol
         |        ,${columns.filter(s=>{s!=s"${cidCol}" && s!=s"new_${cidCol}"}).mkString(",")}
         |FROM    (
         |            SELECT  CONCAT_WS('_',MD5(cleanup(${dupliCols.map(s => s"a.$s").toVector(0)})),MD5(cleanup(${cols_md5.drop(1).map(s => s"a.$s").mkString("")}))) AS rowkey
         |                    ,MD5(cleanup(${dupliCols.map(s => s"a.$s").toVector(0)})) AS new_$cidCol
         |                    ,null AS $cidCol
         |                    ,${columns.filter(s=>{s!=s"${cidCol}" && s!=s"new_${cidCol}"}).map(s => s"a.$s").mkString(",").replace(s"a.${idCardCol}",s"i.identity_num AS ${idCardCol}")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',${dupliCols.map(s => s"a.$s").mkString(",")})) ORDER BY NVL($updateCol,update_time) DESC ) num
         |            FROM    (
         |                        SELECT  ${columns.mkString(",")}
         |                        FROM    ${ods_company_tb}
         |                        WHERE   ds >= ${runDs}
         |                        AND     $cidCol IS NULL
         |                        AND     ${dupliCols.mkString(""," IS NOT NULL AND "," IS NOT NULL")}
         |                    ) a
         |             JOIN
         |                    (
         |                        SELECT *
         |                        FROM ${project}.ads_person_idcard_cloze
         |                        WHERE ds=${personIdDs}
         |                    ) i
         |             ON a.${dupliCols(0)}=i.name AND a.case_no=i.case_no
         |        ) b
         |WHERE   num = 1
         |AND     CONCAT_WS('',${cols_md5.mkString(",")}) IS NOT NULL
         |AND     trim(CONCAT_WS('',${cols_md5.mkString(",")})) <> ''
         |""".stripMargin)
  }



  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    prepareFunctions(spark)

    val ads_company_tb = s"${project}.ads_${tableName}" //存量ads表
    val inc_ods_company_tb = s"${project}.inc_ods_$tableName" //增量ods表
    val inc_ads_company_tb = s"${project}.inc_ads_${tableName}_person" //增量ads表

    //身份证补全表最新分区
    val personIdDs = BaseUtil.getPartion("ads_person_idcard_cloze", spark)

    //存量表ads最新分区
    val remainDs = BaseUtil.getPartion(ads_company_tb, spark)
    //增量ads最后一个分区
    val lastDsIncAds = BaseUtil.getPartion(inc_ads_company_tb, spark)

    val list = sql(s"show partitions $inc_ods_company_tb").collect.toList.map(_.getString(0).split("=")(1))
    //增量ods第一个分区
    val firstDsIncOds = list.head
    //增量ods最后一个分区//落表分区
    val lastDsIncOds = list.last
    //执行分区
    var runDs = ""
    //table字段
    val columns: Seq[String] = spark.table(ads_company_tb).schema.map(_.name).filter(s => {
      !s.equals("ds") && !s.equals(s"new_$cidCol")  && !s.equals("rowkey") && !s.equals("cids")
    })
    //第一次run
    if (StringUtils.isBlank(lastDsIncAds)) {
      runDs = firstDsIncOds
    } else { //非第一次分区时间 + 1天
      runDs = BaseUtil.atDaysAfter(1, lastDsIncAds)
    }

    val cols_md5 = dupliCols.filter(!_.equals(s"new_$cidCol"))

    //增量ods和增量ads最后一个分区相等,跳出
    if (lastDsIncOds.equals(lastDsIncAds)) {
      println("inc_ods equals inc_ads ds ,please delete last ds !!!")
      runDs = lastDsIncOds
      //sys.exit(-1)
    }

    println(
      s"""
         |cols_md5:$cols_md5
         |remainDs:$remainDs
         |lastDsIncOds:$lastDsIncOds
         |lastDsIncAds:$lastDsIncAds
         |runDs:$runDs
         |firstDsIncOds:$firstDsIncOds
         |""".stripMargin)

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${inc_ads_company_tb} PARTITION(ds=$lastDsIncOds)
         |SELECT  rowkey
         |        ,flag
         |        ,new_$cidCol
         |        ,$cidCol
         |        ,${columns.filter(s=>{s!=s"${cidCol}" && s!=s"new_${cidCol}"}).mkString(",")}
         |FROM    (
         |            SELECT  CONCAT_WS('_',MD5(cleanup(${dupliCols.map(s => s"a.$s").toVector(0)})),MD5(cleanup(${cols_md5.drop(1).map(s => s"a.$s").mkString("")}))) AS rowkey
         |                    ,a.flag
         |                    ,MD5(cleanup(${dupliCols.map(s => s"a.$s").toVector(0)})) AS new_$cidCol
         |                    ,null AS $cidCol
         |                    ,${columns.filter(s=>{s!=s"${cidCol}" && s!=s"new_${cidCol}"}).map(s => s"a.$s").mkString(",").replace(s"a.${idCardCol}",s"i.identity_num ${idCardCol}")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',${dupliCols.map(s => s"a.$s").mkString(",")})) ORDER BY NVL($updateCol,update_time) DESC ) num
         |            FROM    (
         |                        SELECT  "1" AS flag
         |                                ,${columns.mkString(",")}
         |                        FROM    ${inc_ods_company_tb}
         |                        WHERE   ds >= ${runDs}
         |                        AND     $cidCol IS NULL
         |                        AND     ${dupliCols.mkString(""," IS NOT NULL AND "," IS NOT NULL")}
         |                    ) a
         |             LEFT JOIN
         |                    (
         |                        SELECT *
         |                        FROM ${project}.ads_person_idcard_cloze
         |                        WHERE ds=${personIdDs}
         |                    ) i
         |             ON a.${dupliCols(0)}=i.name AND a.case_no=i.case_no
         |        ) b
         |WHERE   num = 1
         |AND     CONCAT_WS('',${cols_md5.mkString(",")}) IS NOT NULL
         |AND     trim(CONCAT_WS('',${cols_md5.mkString(",")})) <> ''
         |""".stripMargin)

    val colsTotal = columns ++ Seq(s"new_$cidCol")

    MaxComputer2Phoenix(
      spark,
      colsTotal.filter(!_.equals("cid")).filter(!_.equals("new_cid")).filter(!_.equals("cids")).filter(!_.equals("new_cids")),
      inc_ads_company_tb,
      tableName+"_PERSON",
      lastDsIncOds,
      s"CONCAT_WS('_',new_$cidCol,MD5(cleanup(${cols_md5.drop(1).mkString("")})))"
    ).syn()

//    CompanyIncSummary(spark, project, tableName, "new_cid", dupliCols).calc

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}
object PersonIncrSync {
  def main(args: Array[String]): Unit = {

    val Array(project, tableName, idCardCol, dupliCols, _*) = args
    println(
      s"""
         |project: $project
         |tableName: $tableName
         |idCardCol: $idCardCol
         |dupliCols: $dupliCols
         |""".stripMargin)
    if (!(args.length == 4 || args.length ==6)) {
      println("请输入 project:项目, tableName:表名, idCardCol:身份证字段, dupliCols:去重字段, [cidCol:企业标识, [updateCol:排序列]]!!!")
      sys.exit(-1)
    }
    //兼容zxr表的企业字段为cids;ROW_NUMBER窗口函数的ORDER BY字段，默认（可以不传参数）为update_time
    var cidCol: String ="cid"
    var updateCol: String = "update_time"
    if (args.length >= 5 && !args(4).endsWith("}")) {
      cidCol = args(4)
    }
    if (args.length == 6 && !args(5).endsWith("}")) {
      updateCol = args(5)
    }
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    PersonIncrSync(spark, project, tableName, idCardCol, dupliCols.split(",").toSeq, cidCol, updateCol).calc()
    spark.stop()
  }
}
