package com.winhc.bigdata.spark.jobs.dynamic

import java.util.Date

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.jobs.message.IntellectualMessage
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.ReflectUtils.getClazz
import com.winhc.bigdata.spark.utils.{AsyncExtract, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.time.DateFormatUtils
import org.apache.spark.internal.Logging
import org.apache.spark.sql.types.StringType
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.immutable.ListMap
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/7/27 16:52
 * @Description: 企业动态
 */
object CompanyDynamic {
  val env = "prod"
  val targetTab = "ads_company_dynamic"

  case class CompanyDynamicUtil(s: SparkSession,
                                project: String, //表所在工程名
                                ds: String //此维度主键

                               ) extends LoggingUtils with Logging with BaseFunc {
    @(transient@getter) val spark: SparkSession = s


    def init(): Unit = {
      sql(
        s"""
           |CREATE TABLE IF NOT EXISTS ${getEnvProjectName(env, project)}.$targetTab
           |(
           |    id  STRING COMMENT '唯一标示，cid-sub_info_type-change_time'
           |    ,cid  STRING COMMENT '公司id'
           |    ,cname  STRING COMMENT '公司name'
           |    ,info_type STRING COMMENT '变更分类，大类'
           |    ,rta_desc STRING COMMENT '变更信息描述，变更标题'
           |    ,change_content STRING COMMENT '变更内容'
           |    ,change_time STRING COMMENT '变更时间'
           |    ,biz_id STRING COMMENT '业务id，数据行id'
           |    ,sub_info_type STRING COMMENT '变更小类，表名'
           |    ,info_risk_level STRING COMMENT '变更风险等级'
           |    ,winhc_suggest STRING COMMENT '提示信息'
           |    ,create_time STRING COMMENT '创建时间'
           |)
           |COMMENT '企业动态输出表'
           |PARTITIONED BY (ds STRING COMMENT '分区',tn STRING COMMENT '表名')
           |LIFECYCLE 30
           |""".stripMargin)
    }

    //表名（不加前后辍）
    def calc(tableName: String
             , bName: Int = 0 //是否补充cname字段
            ): Unit = {
      val handle = getClazz[CompanyDynamicHandle](s"com.winhc.bigdata.spark.jobs.dynamic.tables.$tableName")

      val types = handle.org_type()
      val colsExclusiveSome = spark.table(s"${project}.ads_change_extract").columns.filter(s => {
        !s.equals("cid") && !s.equals("data") && !s.equals("old_data") && !s.equals("ds") && !s.equals("tn")
      }).seq
      val rdd = sql(
        bName match {
          //默认:无需补全cname字段
          case 0 =>
            s"""
               |SELECT  *,null AS cname
               |FROM    ${project}.ads_change_extract
               |WHERE   ds = '$ds'
               |AND     tn = '$tableName'
               |AND     TYPE in (${types.map("'" + _ + "'").mkString(",")})
               |""".stripMargin
          //需根据cid补全cname字段数据
          case 1 =>
            s"""
               |SELECT A.*,B.cname AS cname
               |FROM(
               |  SELECT  *
               |  FROM    ${project}.ads_change_extract
               |  WHERE   ds = '$ds'
               |  AND     tn = '$tableName'
               |  AND     TYPE in (${types.map("'" + _ + "'").mkString(",")})
               |) AS A
               |LEFT JOIN (
               |    SELECT cid,cname FROM  $project.base_company_mapping
               |    WHERE ds = '${getLastPartitionsOrElse(project + ".base_company_mapping", "0")}'
               |) AS B
               |ON A.cid = B.cid
               |""".stripMargin
          //既需根据cid补全cname字段数据；还需在data字段中根据其中的cid添加对应cname数据
          case 2 =>
            s"""
               |SELECT cid, ${colsExclusiveSome.mkString(",")},old_data, cname, STR_TO_MAP(regexp_replace(concat_ws(',',data),'[{"}]',''),',',':') AS data
               |FROM(
               |  SELECT new_cid AS cid, ${colsExclusiveSome.mkString(",")},to_json(old_data) AS old_data, COLLECT_SET(cname)[0] AS cname, COLLECT_SET(to_json(data)) AS data
               |  FROM(
               |    SELECT new_cid, ${colsExclusiveSome.map("A." + _).mkString(",")}, A.cid, IF(A.data IS NULL, STR_TO_MAP(CONCAT('cname:',IF(cname IS NULL, '',cname)),',',':'), A.data) AS data, old_data, IF(A.new_cid=A.cid,B.cname,null) AS cname
               |    FROM(
               |      SELECT cid AS new_cid, cid, ${colsExclusiveSome.mkString(",")},data, old_data
               |      FROM(
               |        SELECT  *
               |        FROM    ${project}.ads_change_extract
               |        WHERE   ds = '$ds'
               |        AND     tn = '$tableName'
               |        AND     TYPE in (${types.map("'" + _ + "'").mkString(",")})
               |        )
               |      UNION ALL
               |      SELECT cid AS new_cid, data['cid'] AS cid, ${colsExclusiveSome.mkString(",")},null AS data, old_data
               |      FROM(
               |        SELECT  *
               |        FROM    ${project}.ads_change_extract
               |        WHERE   ds = '$ds'
               |        AND     tn = '$tableName'
               |        AND     TYPE in (${types.map("'" + _ + "'").mkString(",")})
               |        )
               |    ) AS A
               |    LEFT JOIN (
               |        SELECT cid,cname FROM  $project.base_company_mapping
               |        WHERE ds = '${getLastPartitionsOrElse(project + ".base_company_mapping", "0")}'
               |    ) AS B
               |    ON A.cid = B.cid
               |  )
               |  GROUP BY new_cid, ${colsExclusiveSome.mkString(",")},to_json(old_data)
               |)
               |""".stripMargin
        })
        .rdd.flatMap(r => {
        val rowkey = r.getAs[String]("rowkey")
        val cid = r.getAs[String]("cid")
        val new_data = r.getAs[Map[String, String]]("data")
        val old_data = r.getAs[Map[String, String]]("old_data")
        val biz_date = r.getAs[String]("biz_date")
        val fields = r.getAs[String]("fields")
        val cname = r.getAs[String]("cname")
        if (biz_date == null)
          None
        try {
          val result = handle.handle(rowkey, biz_date, cid, if (fields == null) null else fields.split(","), old_data, new_data, cname)
          if (result == null) {
            None
          }
          else {
            result.map(res => Row(CompanyDynamicHandleUtils.getDynamicId(res._1, res._4, res._7, res._6), res._1, res._2, res._3, res._4.replaceAll("null", ""), res._5, res._6, res._7, res._8, res._9, res._10, DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss")))
          }
        } catch {
          case e: Exception => {
            log.error(e.getMessage, e)
            log.error("new_data: {}", new_data)
            log.error("old_data: {}", old_data)
          }
            None
        }
      })

      val schema = getSchema(ListMap(
        "id" -> StringType
        , "cid" -> StringType
        , "cname" -> StringType
        , "info_type" -> StringType
        , "rta_desc" -> StringType
        , "change_content" -> StringType
        , "change_time" -> StringType
        , "biz_id" -> StringType
        , "sub_info_type" -> StringType
        , "info_risk_level" -> StringType
        , "winhc_suggest" -> StringType
        , "create_time" -> StringType
      ))
      spark.createDataFrame(rdd, schema)
        .createOrReplaceTempView(s"company_dynamic_tmp$tableName")

      //      val cols = getColumns(s"$project.$targetTab").filter(!_.equals("ds")).filter(!_.equals("tn"))
      unescapeHtml4()
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${getEnvProjectName(env, project)}.$targetTab PARTITION(ds='$ds',tn='$tableName')
           |SELECT  id
           |        , cid
           |        , cname
           |        , info_type
           |        , unescapeHtml4(rta_desc) rta_desc
           |        , unescapeHtml4(change_content) change_content
           |        , change_time
           |        , biz_id
           |        , sub_info_type
           |        , info_risk_level
           |        , winhc_suggest
           |        , create_time
           |FROM
           |    company_dynamic_tmp$tableName
           |WHERE id IS NOT NULL
           |AND   to_timestamp(change_time) <= now()
           |""".stripMargin)
    }
  }

  private val startArgs = Seq(
    Args(tableName = "company_abnormal_info", bName = 1)
    , Args(tableName = "company_equity_info")
    , Args(tableName = "company_staff", bName = 1)
    , Args(tableName = "company", bName = 0)
    , Args(tableName = "company_illegal_info", bName = 1)
    , Args(tableName = "company_land_publicity", bName = 1)
    , Args(tableName = "company_employment", bName = 1, aggs = 1)
    , Args(tableName = "company_land_announcement", bName = 1)
    , Args(tableName = "company_bid_list", bName = 2)
    , Args(tableName = "company_land_transfer", bName = 1)
    , Args(tableName = "company_land_mortgage", bName = 1)
    , Args(tableName = "company_env_punishment", bName = 1)
    , Args(tableName = "company_punishment_info", bName = 1)
    , Args(tableName = "company_punishment_info_creditchina", bName = 1)
    , Args(tableName = "company_public_announcement2_list", bName = 2)
    , Args(tableName = "bankruptcy_open_case", bName = 1)
    , Args(tableName = "company_mortgage_info", bName = 1)
    , Args(tableName = "company_stock_announcement", bName = 1)
    , Args(tableName = "company_finance", bName = 1)
    , Args(tableName = "company_dishonest_info", bName = 1)
    , Args(tableName = "company_send_announcement_list", bName = 1)
    , Args(tableName = "company_annual_report_out_guarantee", bName = 1)
    , Args(tableName = "company_zxr_restrict", bName = 1)
    , Args(tableName = "company_brief_cancel_announcement", bName = 1)
    , Args(tableName = "company_liquidating_info", bName = 1)


    , Args(tableName = "company_zxr_list", bName = 0) //被执行人
    , Args(tableName = "company_zxr_final_case", bName = 1) //终本案件
    , Args(tableName = "company_license_creditchina", bName = 1) //行政许可-信用中国
    , Args(tableName = "company_license_entpub", bName = 1) //行政许可-企业公示
    , Args(tableName = "company_license", bName = 1) //行政许可
    , Args(tableName = "company_check_info", bName = 1) //抽查检查
    , Args(tableName = "company_court_announcement_list", bName = 2) //法院公告
    , Args(tableName = "company_court_open_announcement_list", bName = 2) //开庭公告
    , Args(tableName = "company_court_register_list", bName = 2) //立案信息
    , Args(tableName = "company_double_random_check_info", bName = 1) //双随机抽查
    , Args(tableName = "company_judicial_sale_combine_list", bName = 1) //司法拍卖
    , Args(tableName = "company_tax_contravention", bName = 1) //税收违法
    , Args(tableName = "wenshu_detail_combine", bName = 1) //裁判文书
    , Args(tableName = "company_holder", bName = 1) //股东
    , Args(tableName = "company_annual_report_out_investment", bName = 1) //裁判文书
    , Args(tableName = "company_own_tax", bName = 1) //欠税公告
    , Args(tableName = "intellectual", bName = 1, aggs = 2) //知识产权
  )

  private case class Args(project: String = "winhc_eci_dev"
                          , tableName: String
                          , bName: Int = 1
                          , aggs: Int = 0)

  def main(args: Array[String]): Unit = {


    if (args.length != 3) {
      println(
        s"""
           |Please enter the legal parameters !
           |<project> <tableNames> <ds>
           |""".stripMargin)
      sys.exit(-99)
    }

    val Array(project, tableNames, ds) = args

    println(
      s"""
         |project: $project
         |tableNames: $tableNames
         |ds: $ds
         |""".stripMargin)

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark = SparkUtils.InitEnv("CompanyDynamic", config)
    val cd = CompanyDynamicUtil(spark, project, ds)
    cd.init()

    var start = startArgs
    if (!tableNames.equals("all")) {
      val set = tableNames.split(",").toSet
      start = start.filter(a => set.contains(a.tableName))
    }

    val a = start.map(e => (e.tableName, () => {
      e.aggs match {
        case 1 => CompanyDynamicForDayCount(spark, project, ds).calc(e.tableName, e.bName) //招聘
        case 2 => IntellectualMessage(spark, ds, project).calc() //知识产权
        case _ => cd.calc(e.tableName, e.bName) //通用处理
      }
      true
    }))

    AsyncExtract.startAndWait(spark, a)


    /*if (tableNames.equals("all")) {
      startArgs.foreach(e => {
        cd.calc(e.tableName, e.bName)
      })
    } else {
      val ts = tableNames.split(",").toSet

    startArgs.filter(e => {
      ts.contains(e.tableName)
    }).foreach(e => {
      if(e.tableName == "company_employment"){
        val cdf = CompanyDynamicForDayCount(spark, project, ds)
        cdf.calc(e.tableName, e.bName)
      }else{
        cd.calc(e.tableName, e.bName)
      }

    })*/
    spark.stop()
  }
}
