package com.winhc.bigdata.spark.jobs.human

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils, TableInitUtil}
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author: XuJiakai
 * @Date: 2020/9/29 11:14
 * @Description: 公司所在维度，只包含人
 */
case class company_judicial_human(s: SparkSession,
                                  project: String //表所在工程名
                                 ) extends LoggingUtils with TableInitUtil with BaseFunc with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  def init(tableName: String): Unit = {
    def ddl(target_tableName: String): Unit = {
      if (!spark.catalog.tableExists(target_tableName)) {
        val ddl = getDDL(org_tableName = s"$project.ods_$tableName", target_tableName = target_tableName, addOtherCols = Seq(("rowkey", "String")))
        println(ddl)
        sys.exit(-1)
      }
    }

    val target_tableName = s"$project.ads_${tableName}_human"
    val inc_target_tableName = s"$project.inc_ads_${tableName}_human"
    ddl(target_tableName)
    ddl(inc_target_tableName)

  }

  def etl(tableName: String, dupliCols: Seq[String], case_no_fields: Seq[String], id_card_field: String = null, is_inc: Boolean = true): Unit = {
    case_no_trim_udf()
    cleanup()
    is_id_card_udf()
    id_card_trim_udf()
    init(tableName)
    val ods_tableName = s"$project.ods_$tableName"
    val inc_ods_tableName = s"$project.inc_ods_$tableName"
    val case_no_f_set = case_no_fields.toSet
    val target_table = s"$project.inc_ads_${tableName}_human"
    val targetCols = getColumns(target_table).filter(f => {
      !f.equals("ds")
    }).map(f => {
      if (case_no_f_set.contains(f)) {
        s"case_no_trim($f) as $f"
      } else if (id_card_field.equals(f)) {
        s"id_card_trim($f) as $f"
      }
      else
        f
    })
    val intersectCols = getColumns(ods_tableName).toSet & getColumns(inc_ods_tableName).toSet

    val cid_name = intersectCols.filter(f => {
      f.equals("cid") || f.equals("cids")
    }).max

    val case_no_f = case_no_fields.map(f => {
      s"AND case_no_trim($f) IS NOT NULL "
    }).mkString(" ")
    val id_card_f = if (id_card_field == null) "" else s" AND is_id_card($id_card_field)"

    def all(): Unit = {
      val last_ds = getLastPartitionsOrElse(ods_tableName, "0")
      val ads_target_table = s"$project.ads_${tableName}_human"

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $ads_target_table PARTITION(ds='$last_ds')
           |SELECT  ${targetCols.mkString(",")}
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds) AS num
           |            FROM    (
           |                        SELECT  MD5(cleanup(concat(${dupliCols.mkString(",")}))) AS rowkey
           |                                ,${intersectCols.mkString(",")}
           |                        FROM    $ods_tableName
           |                        WHERE   ds = '$last_ds'
           |                        AND     $cid_name IS NULL
           |                        AND     trim(cleanup(concat(${dupliCols.mkString(",")}))) <> ''
           |                        $case_no_f
           |                        $id_card_f
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |""".stripMargin)

    }

    def inc(): Unit = {
      val inc_target_table = s"$project.inc_ads_${tableName}_human"
      val inc_last_ds = getLastPartitionsOrElse(inc_ods_tableName, "0")
      val ads_last_ds = getLastPartitionsOrElse(inc_target_table, "0")

      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $inc_target_table PARTITION(ds='$inc_last_ds')
           |SELECT  ${targetCols.mkString(",")}
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds) AS num
           |            FROM    (
           |                        SELECT  MD5(cleanup(concat(${dupliCols.mkString(",")}))) AS rowkey
           |                                ,${intersectCols.mkString(",")}
           |                        FROM    $inc_ods_tableName
           |                        WHERE   ds > '$ads_last_ds'
           |                        AND     $cid_name IS NULL
           |                        AND     trim(cleanup(concat(${dupliCols.mkString(",")}))) <> ''
           |                        $case_no_f
           |                        $id_card_f
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |""".stripMargin)
      import com.winhc.bigdata.spark.implicits.DataFrame2HBaseHelper._

      val write_cols = getColumns(inc_target_table).diff(Seq("rowkey", "ds"))
      sql(
        s"""
           |select *
           |from $inc_target_table
           |where ds = $inc_last_ds
           |""".stripMargin)
        .save2HBase(s"${tableName}_human".toUpperCase, "rowkey", write_cols)
    }

    if (is_inc)
      inc()
    else
      all()
  }
}

object company_judicial_human {
  def main(args: Array[String]): Unit = {
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    company_judicial_human(spark, "winhc_eci_dev")
      .etl("company_dishonest_info", Seq("name", "case_no"), case_no_fields = Seq("case_no", "gist_dd"), id_card_field = "card_num", is_inc = true)

    spark.stop()
  }
}
