package com.winhc.bigdata.spark.ng.change

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.ng.change.NgChangeExtract.getDoubleDataMap
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils._
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging
import org.apache.spark.sql.functions.col
import org.apache.spark.sql.types.{MapType, StringType, StructField, StructType}
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Description: 筛选出数据的具体变更
 */

case class NgChangeExtract(s: SparkSession,
                           project: String, //表所在工程名
                           tableName1: String, //表名（不加前后辍）
                           primaryKey: String, //此维度主键
                           inc_ds: String, //需要计算的分区
                           primaryFields: Seq[String] //主要字段，该字段任意一个不同 则认为发生变化
                           , newlyRegister: Boolean = false
                          ) extends LoggingUtils with Logging {
  @(transient@getter) val spark: SparkSession = s

  val target_tab = "bds_change_extract"
  init()

  def init() {
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS `$project`.`$target_tab` (
         |  `rowkey` STRING COMMENT '该行数据主键',
         |  `company_id` STRING comment '公司id',
         |  `table_name` STRING comment 'hbase表名',
         |  `update_type` STRING comment '数据展示层面的变更类型，insert、update、deleted、other',
         |  `old_data` MAP<STRING,STRING> COMMENT '原数据',
         |  `new_data` MAP<STRING,STRING> COMMENT '新数据',
         |  `change_fields` STRING comment '哪些字段发生变更',
         |  `biz_date` STRING comment '数据变更的时间',
         |  `update_time` STRING comment '当前计算时间')
         | COMMENT '变更动态'
         |PARTITIONED BY (
         |  `ds` STRING COMMENT '时间分区',
         |  `tn` STRING COMMENT '表名分区')
         |""".stripMargin)

    def convert_update_type(update_type: String, company_id: String): String = if (company_id == null) update_type else "create"

    spark.udf.register("convert_update_type", convert_update_type _)

  }


  val updateTimeMapping = Map(
    "wenshu_detail_combine" -> "update_date", //文书排序时间
    "company_equity_info_list" -> "reg_date", //文书排序时间
    "company_lawsuit" -> "pub_date" //文书排序时间
  )
  //不同name映射table
  val tabMapping =
    Map("company_holder_v2" -> "company_holder" //胜诉案件
    )

  //转换字段
  def trans(s: String): String = {
    var res = s
    if (tabMapping.contains(s)) {
      res = tabMapping(s)
    }
    res
  }

  def calc(): Unit = {
    val tableName = trans(tableName1)
    val cols = primaryFields.filter(!_.equals(primaryKey)).seq

    val ds = inc_ds.replace("-", "")

    val intersectCols = getColumns(s"$project.ads_$tableName").toSet & getColumns(s"$project.inc_ads_$tableName").toSet

    val otherAllCols = intersectCols.filter(!primaryKey.equals(_)).toSeq
    val all_cols = primaryKey +: otherAllCols :+ "change_flag"

    val lastDs_ads_all = getLastPartitionsOrElse(s"$project.ads_$tableName", "0")

    val handle = ReflectUtils.getClazz[NgCompanyChangeHandle](s"com.winhc.bigdata.spark.ng.change.table.$tableName1", cols,true)


    val df = sql(
      s"""
         |SELECT  $primaryKey,${otherAllCols.mkString(",")},'0' as change_flag
         |FROM    $project.inc_ads_$tableName
         |WHERE   ds = $ds
         |UNION ALL
         |SELECT  t2.$primaryKey,${otherAllCols.map("t2." + _).mkString(",")},'1' as change_flag
         |FROM    (
         |            SELECT  DISTINCT ${primaryKey}
         |            FROM    $project.inc_ads_$tableName
         |            WHERE   ds = $ds
         |        ) AS t1
         |JOIN    (
         |             SELECT  tmp.*
         |             FROM    (
         |                         SELECT  a.*
         |                                 ,row_number() OVER (PARTITION BY a.${primaryKey} ORDER BY ds DESC, ${updateTimeMapping.getOrElse(tableName, "update_time")} DESC) c
         |                         FROM    (
         |                                     SELECT  ${intersectCols.mkString(",")},ds
         |                                     FROM    $project.ads_$tableName
         |                                     WHERE   ds = $lastDs_ads_all
         |                                     UNION ALL
         |                                     SELECT  ${intersectCols.mkString(",")},ds
         |                                     FROM    $project.inc_ads_$tableName
         |                                     WHERE   ds > $lastDs_ads_all and ds < $ds
         |                                 ) AS a
         |                     ) AS tmp
         |             WHERE   tmp.c = 1
         |        ) AS t2
         |ON      t1.${primaryKey} = t2.${primaryKey}
         |""".stripMargin)


    val rdd =
      df.select(all_cols.map(column => col(column).cast("string")): _*)
        .rdd.map(r => {
        (r.getAs[String](primaryKey), all_cols.map(f => (f, r.getAs[String](f))).toMap)
      }).groupByKey()
        .map(x => {
          val rowkey = x._1
          val map_list = x._2
          val m = getDoubleDataMap(map_list)

          val new_map = m._1
          val old_map = m._2
          val res = handle.handle(rowkey, old_map, new_map)
          if (res == null) {
            null
          } else {
            val rowkey = res._1
            val company_id = res._2
            val update_type = res._3
            val old_map = res._4
            val new_map = res._5
            val change_fields = res._6
            val biz_date = res._7
            val update_time = BaseUtil.nowDate()

            Row(rowkey, company_id, tableName, update_type, old_map, new_map, change_fields, biz_date, update_time)
          }
        }).filter(_ != null)

    val schema = StructType(Array(
      StructField("rowkey", StringType), //表数据主建
      StructField("company_id", StringType), //公司id
      StructField("table_name", StringType), //表名
      StructField("update_type", StringType), // 变更类型 insert update
      StructField("old_data", MapType(StringType, StringType)), //变更前数据
      StructField("new_data", MapType(StringType, StringType)), //变更后数据
      StructField("change_fields", StringType), //如果是更新 则显示更新字段
      StructField("biz_date", StringType), //业务时间
      StructField("update_time", StringType) //处理时间
    ))

    spark.createDataFrame(rdd, schema)
      .createOrReplaceTempView(s"tmp_change_extract_view_$tableName1")

    if ("company".equals(tableName1) || !newlyRegister) {
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${project}.$target_tab PARTITION(ds='$ds',tn='$tableName1')
           |SELECT *
           |FROM
           |    tmp_change_extract_view_$tableName1
           |""".stripMargin)
    } else {
      def wait: Boolean = {
        val last_ds = getLastPartitionsOrElse(s"${project}.$target_tab", null, expression = "tn=company")
        !ds.equals(last_ds)
      }
      //等待公司基本信息完成识别
      while (wait) {
        logInfo("wait company tab 。。。")
        println("wait company tab 。。。")
        Thread.sleep(100000)
      }
      sql(
        s"""
           |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${project}.$target_tab PARTITION(ds='$ds',tn='$tableName1')
           |SELECT  t1.rowkey
           |        ,t1.company_id
           |        ,t1.table_name
           |        ,convert_update_type(t1.update_type,t2.company_id) as update_type
           |        ,t1.old_data
           |        ,t1.new_data
           |        ,t1.change_fields
           |        ,t1.biz_date
           |        ,t1.update_time
           |FROM    tmp_change_extract_view_$tableName1 AS t1
           |LEFT JOIN (
           |              SELECT  *
           |              FROM    winhc_ng.bds_change_extract
           |              WHERE   ds = '$ds'
           |              AND     tn = 'company'
           |              AND     update_type = 'insert'
           |          ) AS t2
           |ON      t1.company_id = t2.company_id
           |""".stripMargin)
    }
  }
}


object NgChangeExtract {

  //判断两个map在指定key上是否相等，如不等反回不相等字段
  def getDoubleDataMap(iterable: Iterable[Map[String, String]]): (Map[String, String], Map[String, String]) = {
    val map = iterable.map(m => (m("change_flag"), m)).toMap
    (map.getOrElse("0", null), map.getOrElse("1", null))
  }


  def main(args: Array[String]): Unit = {
    val Array(tableName, inc_ds) = args
    if (args.size != 2) {
      println("please set tableName ds.")
      sys.exit(-1)
    }
    println(s"table : $tableName , ds : $inc_ds")
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv("NgChangeExtract", config)

    var start = NgChangeExtractArgs.startArgs
    if (!tableName.equals("all")) {
      val set = tableName.split(",").toSet
      start = start.filter(a => set.contains(a.tableName))
    }

    val a = start.map(e => (e.tableName, () => {
      val primaryFields: Seq[String] = if (StringUtils.isNotEmpty(e.primaryFields)) e.primaryFields.split(",") else Seq.empty[String]
      NgChangeExtract(spark, e.project, e.tableName, e.primaryKey, inc_ds, primaryFields, newlyRegister = e.newlyRegister).calc()
      true
    }))

    AsyncExtract.startAndWait(spark, a)

    spark.stop()
  }
}
