package com.winhc.bigdata.spark.ng.credit_punishment

import com.winhc.bigdata.spark.ng.credit_punishment.udf.CreditPunishmentCaseAggUDF
import com.winhc.bigdata.spark.ng.utils.StartAndEndDsUtils
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/5/10 18:23
 *
 *        根据案件聚合
 */
case class CreditPunishmentCaseAgg(s: SparkSession
                                  ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = CreditPunishment.caseTargetTab

  init()

  private def init(): Unit = {
    spark.udf.register("credit_punishment_case_agg", new CreditPunishmentCaseAggUDF)

    def generate_entity_id(keyno: String, card_num: String, person_id: String, credit_punishment_case_id: String): String = {

      if (StringUtils.isNotEmpty(keyno) && keyno.length == 32) {
        return keyno
      }
      if (BaseUtil.is_id_card(card_num)) {
        return person_id
      }
      credit_punishment_case_id
    }

    spark.udf.register("generate_entity_id", generate_entity_id _)

    def keyno_type(keyno: String): String = if (keyno == null) "1" else if (keyno.length == 32) "0" else "1"
    spark.udf.register("keyno_type", keyno_type _)

    cleanup()
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    credit_punishment_case_id STRING COMMENT '信用惩戒查询案件id'
         |    ,credit_punishment_entity_id STRING COMMENT '信用惩戒对象主体id'
         |    ,name STRING COMMENT '惩戒主体名称'
         |    ,court_name STRING COMMENT '法院名称'
         |    ,case_no STRING COMMENT '案号'
         |    ,rowkey STRING COMMENT '涉案rowkey'
         |    ,card_num STRING COMMENT '案号'
         |    ,keyno STRING COMMENT '惩戒主体id，公司id或人id'
         |    ,keyno_type STRING COMMENT '惩戒主体类型，0 公司 1人'
         |    ,label STRING COMMENT '标签'
         |    ,case_create_time STRING COMMENT '立案时间'
         |    ,record_num bigint COMMENT '记录条数'
         |
         |    ,exec_info string COMMENT '执行情况'
         |    ,amount_involved string COMMENT '涉案金额'
         |
         |    ,total_exec_amount STRING COMMENT '累计被执行总金额'
         |    ,total_no_exec_amount STRING COMMENT '疑似当前欠款总金额'
         |    ,zxr_total_exec_amount STRING COMMENT '被执行人当前被执行总金额'
         |    ,final_case_exec_amount STRING COMMENT '终本案件执行标的总金额'
         |    ,final_case_no_exec_amount STRING COMMENT '终本案件未履行总金额'
         |
         |    ,company_dishonest_info_num_0 BIGINT COMMENT '失信人条数'
         |    ,company_zxr_num_0 BIGINT COMMENT '被执条数'
         |    ,company_zxr_final_case_num_0 BIGINT COMMENT '终本案件条数'
         |    ,company_zxr_restrict_num_0 BIGINT COMMENT '限高条数'
         |
         |    ,company_dishonest_info_num_1 BIGINT COMMENT '失信人条数-历史'
         |    ,company_zxr_num_1 BIGINT COMMENT '被执条数-历史'
         |    ,company_zxr_final_case_num_1 BIGINT COMMENT '终本案件条数-历史'
         |    ,company_zxr_restrict_num_1 BIGINT COMMENT '限高条数-历史'
         |
         |    ,deleted BIGINT COMMENT '0：未删除，1：删除'
         |    ,all_rowkey STRING COMMENT '所有rowkey的集合，包含历史非历史'
         |)
         |COMMENT '信用惩戒查询案件表'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)

  }


  private def getCreditPunishmentDataExtraction(ds: String): String = {
    val tmp_view = "credit_punishment_data_extraction_calc"

    //去重 tn + rowkey + name
    //关联 (rowkey + tn) or (case_no + court_name)

    if (ds == null) {
      sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY tn,rowkey,name ORDER BY ds DESC) AS num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    ${CreditPunishment.extractionTargetTab}
           |                        WHERE   ds > '0'
           |                    )
           |        )
           |WHERE   num = 1
           |AND     court_name is not null
           |AND     trim(court_name) <> ''
           |""".stripMargin)
        .createTempView(tmp_view)
    } else {
      sql(
        s"""
           |SELECT  *
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY tn,rowkey,name ORDER BY ds DESC) AS num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    ${CreditPunishment.extractionTargetTab}
           |                        WHERE   ds > '$ds'
           |                        UNION ALL
           |                        SELECT  t2.*
           |                        FROM    (
           |                                    SELECT  *
           |                                    FROM    winhc_ng.bds_credit_punishment_data_extraction
           |                                    WHERE   ds > '$ds'
           |                                ) AS t1
           |                        JOIN    (
           |                                    SELECT  *
           |                                    FROM    winhc_ng.bds_credit_punishment_data_extraction
           |                                    WHERE   ds <= '$ds'
           |                                ) as t2
           |                        ON      t1.case_no = t2.case_no
           |                        AND     t1. name = t2.name
           |                        AND     t1.court_name = t2. court_name
           |                    )
           |        )
           |WHERE   num = 1
           |AND     court_name is not null
           |AND     trim(court_name) <> ''
           |""".stripMargin)
        .createTempView(tmp_view)
    }
    tmp_view
  }


  private def getAllDataByTmpData(tmpTab: String): String = {

    val cols = getColumns(CreditPunishment.caseTargetTab)
    sql(
      s"""
         |SELECT
         |        split(xjk_rowkey,'@@') [0 ] AS tn
         |        ,split(xjk_rowkey,'@@') [2 ] AS rowkey
         |        ,name
         |FROM    (
         |        SELECT
         |                *
         |                ,ROW_NUMBER() OVER(PARTITION BY xjk_rowkey,name ORDER BY xjk_rowkey DESC ) AS num
         |        FROM    (
         |                SELECT  *
         |                FROM    (
         |                        SELECT  t2.*
         |                        FROM    (
         |                                SELECT DISTINCT  credit_punishment_case_id
         |                                FROM    ${tmpTab}
         |                                ) AS t1
         |                        JOIN (
         |                                SELECT  *
         |                                FROM    ${CreditPunishment.caseTargetTab}
         |                                WHERE   ds > '0'
         |                             ) AS t2
         |                        ON      t1.credit_punishment_case_id = t2.credit_punishment_case_id
         |                        UNION ALL
         |                        SELECT  ${cols.diff(Seq("ds")).mkString(",")},'${BaseUtil.getYesterday()}' as ds
         |                        FROM    ${tmpTab}
         |                        UNION ALL
         |                        SELECT  tt2.*
         |                        FROM    (
         |                                SELECT DISTINCT  credit_punishment_entity_id
         |                                FROM    ${tmpTab}
         |                                ) AS tt1
         |                        JOIN (
         |                                SELECT  *
         |                                FROM    ${CreditPunishment.caseTargetTab}
         |                                WHERE   ds > '0'
         |                             ) AS tt2
         |                        ON      tt1.credit_punishment_entity_id = tt2.credit_punishment_entity_id
         |                        ) AS tab3
         |                LATERAL VIEW explode(split(all_rowkey,',')) tt AS xjk_rowkey )
         |        )
         |WHERE   num = 1
         |
         |""".stripMargin)
      .createTempView("all_rowkey")

    sql(
      s"""
         |SELECT  *
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey,tn,name ORDER BY ds DESC) AS num
         |            FROM    (
         |                        SELECT  t1.*
         |                        FROM    (
         |                                    SELECT  *
         |                                    FROM    winhc_ng.bds_credit_punishment_data_extraction
         |                                    WHERE   ds > '0'
         |                                ) AS t1
         |                        JOIN    all_rowkey AS t2
         |                        ON      t1.rowkey = t2.rowkey
         |                        AND     t1.tn = t2.tn
         |                        AND     t1.name = t2.name
         |                    )
         |        )
         |WHERE   num = 1
         |""".stripMargin)

      .createTempView("all_record")
    "all_record"
  }

  private def get_sql(org_tab: String): String = {
    s"""
       |SELECT  md5(cleanup(CONCAT_WS('',name,court_name,case_no))) AS credit_punishment_case_id
       |        ,generate_entity_id(mm['keyno'],mm['card_num'],md5(CONCAT_WS('',mm['card_num'],name)),md5(cleanup(CONCAT_WS('',name,court_name,case_no)))) as credit_punishment_entity_id
       |        ,name
       |        ,court_name
       |        ,case_no
       |        ,mm['rowkey'] AS rowkey
       |        ,mm['card_num'] AS card_num
       |        ,mm['keyno'] AS keyno
       |        ,keyno_type(mm['keyno']) AS keyno_type
       |        ,mm['label'] AS label
       |        ,mm['case_create_time'] AS case_create_time
       |        ,cast(mm['record_num'] as bigint) AS record_num
       |
       |        ,mm['exec_info'] AS exec_info
       |        ,mm['amount_involved'] AS amount_involved
       |
       |        ,mm['total_exec_amount'] AS total_exec_amount
       |        ,mm['total_no_exec_amount'] AS total_no_exec_amount
       |        ,mm['zxr_total_exec_amount'] AS zxr_total_exec_amount
       |        ,mm['final_case_exec_amount'] AS final_case_exec_amount
       |        ,mm['final_case_no_exec_amount'] AS final_case_no_exec_amount
       |
       |        ,CAST(mm['company_dishonest_info_num_0'] AS BIGINT) AS company_dishonest_info_num_0
       |        ,CAST(mm['company_zxr_num_0'] AS BIGINT) AS company_zxr_num_0
       |        ,CAST(mm['company_zxr_final_case_num_0'] AS BIGINT) AS company_zxr_final_case_num_0
       |        ,CAST(mm['company_zxr_restrict_num_0'] AS BIGINT) AS company_zxr_restrict_num_0
       |
       |        ,CAST(mm['company_dishonest_info_num_1'] AS BIGINT) AS company_dishonest_info_num_1
       |        ,CAST(mm['company_zxr_num_1'] AS BIGINT) AS company_zxr_num_1
       |        ,CAST(mm['company_zxr_final_case_num_1'] AS BIGINT) AS company_zxr_final_case_num_1
       |        ,CAST(mm['company_zxr_restrict_num_1'] AS BIGINT) AS company_zxr_restrict_num_1
       |
       |        ,cast(mm['deleted'] as bigint) AS deleted
       |        ,mm['all_rowkey'] AS all_rowkey
       |FROM    (
       |            SELECT  collect_set(name)[0] as name
       |                    ,collect_set(court_name)[0] as court_name
       |                    ,collect_set(case_no)[0] as case_no
       |                    ,credit_punishment_case_agg(rowkey,tn,keyno,card_num,case_create_time,deleted,detail_data) AS mm
       |            FROM    $org_tab
       |            GROUP BY cleanup(concat_ws('',name,court_name,case_no))
       |        )
       |WHERE   name IS NOT NULL and trim(name) <> ''
       |""".stripMargin
  }


  def calc(ds: String): Unit = {
    val args = StartAndEndDsUtils(spark).get_gt_ds(CreditPunishment.extractionTargetTab, CreditPunishment.caseTargetTab)

    val org_tab = getCreditPunishmentDataExtraction(args.inc_tab_gt_ds)


    sql(
      s"""
         |${get_sql(org_tab)}
         |""".stripMargin)
      .cache()
      .createTempView(s"tmp_data1")

    val all_record = getAllDataByTmpData("tmp_data1")

    //test
   /* sql(
      s"""
         |CREATE TABLE IF NOT EXISTS winhc_ng.xjk_test_01 as
         |select * from
         |tmp_data1
         |""".stripMargin)

    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS winhc_ng.xjk_test_02 as
         |select * from
         |$all_record
         |""".stripMargin)
    return*/
    //test end

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $target_tab PARTITION(ds='${ds}')
         |${get_sql(all_record)}
         |""".stripMargin)
  }
}
