package com.winhc.bigdata.spark.ng.credit_punishment

import com.winhc.bigdata.spark.ng.utils.StartAndEndDsUtils
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.LoggingUtils
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions.{col, struct, to_json}

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/5/10 09:48
 *        step.01 四个维度数据抽取
 *
 */
case class CreditPunishmentDataExtraction(s: SparkSession
                                         ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s


  private val target_tab = CreditPunishment.extractionTargetTab
  init()

  private def init(): Unit = {

    def case_no_replace(case_no: String): String = if (StringUtils.isEmpty(case_no)) null else case_no.replace("（", "(").replace("）", ")").trim

    spark.udf.register("case_no_replace", case_no_replace _)

    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    rowkey STRING COMMENT 'hbase中rowkey'
         |    ,tn STRING COMMENT '数据维度'
         |    ,keyno STRING COMMENT '公司id或人的id'
         |    ,name STRING COMMENT '公司名或人名'
         |    ,card_num STRING COMMENT '身份证号或工商注册号'
         |    ,court_name STRING COMMENT '法院名称'
         |    ,case_no STRING COMMENT '案号'
         |    ,case_create_time datetime COMMENT '立案时间'
         |    ,deleted BIGINT  COMMENT '是否删除'
         |    ,detail_data STRING COMMENT '祥情数据'
         |)
         |COMMENT 'TABLE COMMENT'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)
  }


  private def getData(tab: String, ds: String): String = {
    val ads_tab = s"winhc_ng.ads_$tab"
    val inc_ads_tab = s"winhc_ng.inc_ads_$tab"
    val cols = getColumns(ads_tab)

    if (ds == null) {
      sql(
        s"""
           |SELECT  ${cols.mkString(",")}
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    $ads_tab
           |                        WHERE   ds > 0
           |                        UNION ALL
           |                        SELECT  *
           |                        FROM    $inc_ads_tab
           |                        WHERE   ds > 0
           |                    )
           |        )
           |WHERE   num = 1
           |""".stripMargin)
        .createTempView(s"${tab}_all_data")
    } else {
      sql(
        s"""
           |SELECT  ${cols.mkString(",")}
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  *
           |                        FROM    $inc_ads_tab
           |                        WHERE   ds > '$ds'
           |                    )
           |        )
           |WHERE   num = 1
           |""".stripMargin)
        .createTempView(s"${tab}_all_data")
    }
    s"${tab}_all_data"
  }


  private def dataExtraction(ds: String, target_ds: String): Unit = {
    val company_dishonest_info = getData("company_dishonest_info", ds)
    val company_zxr = getData("company_zxr", ds)
    val company_zxr_final_case = getData("company_zxr_final_case", ds)
    val company_zxr_restrict = getData("company_zxr_restrict", ds)


    val company_dishonest_info_cols = getColumns(s"winhc_ng.ads_company_dishonest_info")
    val company_zxr_cols = getColumns(s"winhc_ng.ads_company_zxr")
    val company_zxr_final_case_cols = getColumns(s"winhc_ng.ads_company_zxr_final_case")
    val company_zxr_restrict_cols = getColumns(s"winhc_ng.ads_company_zxr_restrict")


    val company_dishonest_info_df = sql(
      s"""
         |SELECT rowkey as xjk_rowkey
         |       ,'company_dishonest_info' as tn
         |       ,keyno as xjk_keyno
         |       ,name as xjk_name
         |       ,card_num as xjk_card_num
         |       ,court as xjk_court_name
         |       ,case_no as xjk_case_no
         |       ,reg_time as xjk_case_create_time
         |       ,deleted as xjk_deleted
         |       ,*
         |from   $company_dishonest_info
         |""".stripMargin)
      .withColumn("detail_data", to_json(struct(company_dishonest_info_cols.map(col): _*)))
      .select("xjk_rowkey"
        , "tn"
        , "xjk_keyno"
        , "xjk_name"
        , "xjk_card_num"
        , "xjk_court_name"
        , "xjk_case_no"
        , "xjk_case_create_time"
        , "xjk_deleted"
        , "detail_data"
      )


    val company_zxr_df = sql(
      s"""
         |SELECT rowkey as xjk_rowkey
         |       ,'company_zxr' as tn
         |       ,keyno as xjk_keyno
         |       ,name as xjk_name
         |       ,card as xjk_card_num
         |       ,court as xjk_court_name
         |       ,case_no as xjk_case_no
         |       ,case_create_time as xjk_case_create_time
         |       ,deleted as xjk_deleted
         |       ,*
         |from   $company_zxr
         |""".stripMargin)
      .withColumn("detail_data", to_json(struct(company_zxr_cols.map(col): _*)))
      .select("xjk_rowkey"
        , "tn"
        , "xjk_keyno"
        , "xjk_name"
        , "xjk_card_num"
        , "xjk_court_name"
        , "xjk_case_no"
        , "xjk_case_create_time"
        , "xjk_deleted"
        , "detail_data"
      )

    val company_zxr_final_case_df = sql(
      s"""
         |SELECT rowkey as xjk_rowkey
         |       ,'company_zxr_final_case' as tn
         |       ,keyno as xjk_keyno
         |       ,name as xjk_name
         |       ,identity_num as xjk_card_num
         |       ,court_name as xjk_court_name
         |       ,case_no as xjk_case_no
         |       ,case_create_time as xjk_case_create_time
         |       ,deleted as xjk_deleted
         |       ,*
         |from   $company_zxr_final_case
         |""".stripMargin)
      .withColumn("detail_data", to_json(struct(company_zxr_final_case_cols.map(col): _*)))
      .select("xjk_rowkey"
        , "tn"
        , "xjk_keyno"
        , "xjk_name"
        , "xjk_card_num"
        , "xjk_court_name"
        , "xjk_case_no"
        , "xjk_case_create_time"
        , "xjk_deleted"
        , "detail_data"
      )

    val company_zxr_restrict_df = sql(
      s"""
         |SELECT rowkey as xjk_rowkey
         |       ,'company_zxr_restrict' as tn
         |       ,pid as xjk_keyno
         |       ,person_name as xjk_name
         |       ,coalesce(card_num,identity_num) as xjk_card_num
         |       ,court_name as xjk_court_name
         |       ,case_no as xjk_case_no
         |       ,case_create_time as xjk_case_create_time
         |       ,deleted as xjk_deleted
         |       ,*
         |from   $company_zxr_restrict
         |
         |UNION ALL
         |
         |SELECT rowkey as xjk_rowkey
         |       ,'company_zxr_restrict' as tn
         |       ,company_id as xjk_keyno
         |       ,company_name as xjk_name
         |       ,coalesce(card_num,identity_num) as xjk_card_num
         |       ,court_name as xjk_court_name
         |       ,case_no as xjk_case_no
         |       ,case_create_time as xjk_case_create_time
         |       ,deleted as xjk_deleted
         |       ,*
         |from   $company_zxr_restrict
         |""".stripMargin)
      .withColumn("detail_data", to_json(struct(company_zxr_restrict_cols.map(col): _*)))
      .select("xjk_rowkey"
        , "tn"
        , "xjk_keyno"
        , "xjk_name"
        , "xjk_card_num"
        , "xjk_court_name"
        , "xjk_case_no"
        , "xjk_case_create_time"
        , "xjk_deleted"
        , "detail_data"
      )


    val df = company_dishonest_info_df
      .union(company_zxr_df)
      .union(company_zxr_final_case_df)
      .union(company_zxr_restrict_df)

    df.createOrReplaceTempView(s"credit_punishment_tmp_view")
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE ${target_tab} PARTITION (ds='$target_ds')
         |SELECT xjk_rowkey as rowkey
         |       ,tn as tn
         |       ,xjk_keyno as keyno
         |       ,trim(xjk_name) as name
         |       ,xjk_card_num as card_num
         |       ,trim(xjk_court_name) as court_name
         |       ,case_no_replace(xjk_case_no) as case_no
         |       ,xjk_case_create_time as case_create_time
         |       ,xjk_deleted as deleted
         |       ,detail_data
         |FROM   credit_punishment_tmp_view
         |WHERE  xjk_court_name is not null
         |AND    trim(xjk_court_name) <> ''
         |""".stripMargin)
  }


  def calc(): Unit = {
    val c = StartAndEndDsUtils(spark).get_start_and_end_args("winhc_ng.ads_company_dishonest_info", "winhc_ng.inc_ads_company_dishonest_info", target_tab)
    dataExtraction(c.inc_tab_gt_ds, c.target_ds)
  }

}
