package com.winhc.bigdata.spark.ng.credit_punishment

import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.ng.credit_punishment.udf.DeletedMergeUDF
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{DateUtils, LoggingUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/5/11 17:41
 *        信用惩戒查询主体聚合
 */
case class CreditPunishmentEntityAgg(s: SparkSession
                                    ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = CreditPunishment.entityTargetTab
  init()

  case class person_other(birth_year: String, gender: String)

  case class company_other(legal_entity_id: String, legal_entity_name: String, legal_entity_type: String, reg_capital: String, reg_capital_amount: String, reg_capital_currency: String, estiblish_time: String, logo: String)

  private def init(): Unit = {
    areaCode2Name_pro()
    double_2_str()

    def array_distinct(arr: Seq[String]): String = if (arr == null || arr.isEmpty) null else arr.filter(StringUtils.isNotEmpty).distinct.mkString(",")

    spark.udf.register("array_distinct", array_distinct _)
    spark.udf.register("deleted_merge", new DeletedMergeUDF)

    def toTime(str: String): String = DateUtils.toMillisTimestamp(str, pattern = "yyyy-MM-dd HH:mm:ss")

    spark.udf.register("to_millis_timestamp", toTime _)


    def get_rowkey_by_prefix(rs: String, prefix: String): String = {
      if (StringUtils.isEmpty(rs)) {
        null
      } else {
        rs.split(",").filter(r => r.split("@@")(0).equals(prefix))
          .map(_.split("@@")(2))
          .toSeq.mkStringOrNull()
      }
    }

    spark.udf.register("get_rowkey_by_prefix", get_rowkey_by_prefix _)

    def get_gender(card_num: String): Int = {
      if (StringUtils.isEmpty(card_num)) {
        return -1
      }
      card_num.substring(16, 17).toInt % 2
    }

    def get_birth_year(card_num: String) = card_num.substring(6, 10)


    def company_other_info(legal_entity_id: String, legal_entity_name: String, legal_entity_type: String, reg_capital: String, reg_capital_amount: String, reg_capital_currency: String, estiblish_time: String, logo: String): String =
      company_other(legal_entity_id = legal_entity_id, legal_entity_name = legal_entity_name, legal_entity_type = legal_entity_type, reg_capital = reg_capital, reg_capital_amount = reg_capital_amount, reg_capital_currency = reg_capital_currency, estiblish_time = estiblish_time, logo = logo).toJson()


    def person_other_info(birth_year: String, gender: String): String =
      person_other(birth_year = birth_year, gender = gender).toJson()

    spark.udf.register("company_other_info", company_other_info _)
    spark.udf.register("person_other_info", person_other_info _)

    spark.udf.register("get_gender", get_gender _)
    spark.udf.register("get_birth_year", get_birth_year _)


    def deleted_person_data(entity_type: Long, name: String, card_num: String, deleted: Long): Long = {
      try {
        if (entity_type == 1 && card_num == null) {
          if (name.contains("公司")) 9
          else deleted
        } else
          deleted
      } catch {
        case ex: Exception => {
          deleted
        }
      }
    }

    spark.udf.register("deleted_person_data", deleted_person_data _)

    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    credit_punishment_entity_id STRING COMMENT '信用惩戒对象主体id'
         |    ,keyno STRING COMMENT '惩戒主体id，公司id或人id'
         |    ,name STRING COMMENT '惩戒主体名称'
         |    ,entity_type bigint COMMENT '0为公司，1为人'
         |    ,card_num STRING COMMENT '身份证号或工商注册号'
         |    ,label STRING COMMENT '标签'
         |    ,province STRING COMMENT '省'
         |    ,city STRING COMMENT '市'
         |    ,county STRING COMMENT '区'
         |    ,other_info STRING COMMENT '其他信息-公司或人的其它信息，json格式'
         |    ,record_num bigint COMMENT '记录条数'
         |
         |    ,create_case_time_year string COMMENT '立案时间，年份，逗号分割'
         |
         |    ,total_exec_amount STRING COMMENT '累计被执行总金额'
         |    ,total_no_exec_amount STRING COMMENT '疑似当前欠款总金额'
         |    ,zxr_total_exec_amount STRING COMMENT '被执行人当前被执行总金额'
         |    ,final_case_exec_amount STRING COMMENT '终本案件执行标的总金额'
         |    ,final_case_no_exec_amount STRING COMMENT '终本案件未履行总金额'
         |
         |    ,company_dishonest_info_num_0 BIGINT COMMENT '失信人条数'
         |    ,company_zxr_num_0 BIGINT COMMENT '被执条数'
         |    ,company_zxr_final_case_num_0 BIGINT COMMENT '终本案件条数'
         |    ,company_zxr_restrict_num_0 BIGINT COMMENT '限高条数'
         |
         |    ,company_dishonest_info_num_1 BIGINT COMMENT '失信人条数-历史'
         |    ,company_zxr_num_1 BIGINT COMMENT '被执条数-历史'
         |    ,company_zxr_final_case_num_1 BIGINT COMMENT '终本案件条数-历史'
         |    ,company_zxr_restrict_num_1 BIGINT COMMENT '限高条数-历史'
         |
         |    ,deleted BIGINT COMMENT '0：未删除，1：删除'
         |
         |    ,company_dishonest_info_rowkey String COMMENT '对应维度rowkey'
         |    ,company_zxr_rowkey String COMMENT '对应维度rowkey'
         |    ,company_zxr_final_case_rowkey String COMMENT '对应维度rowkey'
         |    ,company_zxr_restrict_rowkey String COMMENT '对应维度rowkey'
         |)
         |COMMENT '信用惩戒查询-惩戒主体表'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)
  }

  private def getAllCompany(): String = {
    sql(
      s"""
         |SELECT  *,concat_ws('',province_code,city_code,county_code) as area_code
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY company_id ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  company_id,new_cid,base,name,name_en,name_alias,history_names,legal_entity_id,legal_entity_name,legal_entity_type,reg_number,company_org_type,reg_location,estiblish_time,from_time,to_time,business_scope,reg_institute,approved_time,reg_status,reg_capital,org_number,org_approved_institute,parent_company_id,company_type,credit_code,score,cate_first_code,cate_second_code,cate_third_code,lat,lng,province_code,city_code,county_code,reg_capital_amount,reg_capital_currency,actual_capital_amount,actual_capital_currency,reg_status_std,social_security_staff_num,cancel_date,cancel_reason,revoke_date,revoke_reason,emails,phones,wechat_public_num,logo,crawled_time,create_time,update_time,deleted,ds
         |                        FROM    winhc_ng.ads_company
         |                        WHERE   ds > 0
         |                        UNION ALL
         |                        SELECT  company_id,new_cid,base,name,name_en,name_alias,history_names,legal_entity_id,legal_entity_name,legal_entity_type,reg_number,company_org_type,reg_location,estiblish_time,from_time,to_time,business_scope,reg_institute,approved_time,reg_status,reg_capital,org_number,org_approved_institute,parent_company_id,company_type,credit_code,score,cate_first_code,cate_second_code,cate_third_code,lat,lng,province_code,city_code,county_code,reg_capital_amount,reg_capital_currency,actual_capital_amount,actual_capital_currency,reg_status_std,social_security_staff_num,cancel_date,cancel_reason,revoke_date,revoke_reason,emails,phones,wechat_public_num,logo,crawled_time,create_time,update_time,deleted,ds
         |                        FROM    winhc_ng.inc_ads_company
         |                        WHERE   ds > 0
         |                    ) AS all_t1
         |        ) AS all_t2
         |WHERE   all_t2.num = 1
         |""".stripMargin)
      .createTempView("all_company")
    "all_company"
  }


  private def getCaseInfo(ds: String): String = {
    val tmp_view = "credit_punishment_case_info_calc"
    sql(
      s"""
         |SELECT  *
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY credit_punishment_case_id ORDER BY ds DESC) AS num
         |            FROM    (
         |                        SELECT  t2.*
         |                        FROM    (
         |                                    SELECT  DISTINCT credit_punishment_case_id
         |                                    FROM    ${CreditPunishment.caseTargetTab}
         |                                    WHERE   ds = '$ds'
         |                                ) AS t1
         |                        JOIN    (
         |                                    SELECT  *
         |                                    FROM    ${CreditPunishment.caseTargetTab}
         |                                    WHERE   ds < '$ds'
         |                                ) AS t2
         |                        ON      t1.credit_punishment_case_id = t2.credit_punishment_case_id
         |                        UNION ALL
         |                        SELECT  *
         |                        FROM    ${CreditPunishment.caseTargetTab}
         |                        WHERE   ds = '$ds'
         |                        UNION ALL
         |                        SELECT  tt2.*
         |                        FROM    (
         |                                    SELECT  DISTINCT credit_punishment_entity_id
         |                                    FROM    ${CreditPunishment.caseTargetTab}
         |                                    WHERE   ds = '$ds'
         |                                ) AS tt1
         |                        JOIN    (
         |                                    SELECT  *
         |                                    FROM    ${CreditPunishment.caseTargetTab}
         |                                    WHERE   ds < '$ds'
         |                                ) AS tt2
         |                        ON      tt1.credit_punishment_entity_id = tt2.credit_punishment_entity_id
         |                    )
         |        )
         |WHERE   num = 1
         |AND     deleted <> 9
         |""".stripMargin)
      .createTempView(tmp_view)

    tmp_view
  }


  def calc(ds: String): Unit = {
    val org_tab = getCaseInfo(ds)
    val all_company = getAllCompany()

    sql(
      s"""
         |select credit_punishment_entity_id
         |       ,keyno
         |       ,name
         |       ,card_num
         |       ,label
         |       ,record_num
         |       ,case_create_time_year
         |       ,total_exec_amount
         |       ,total_no_exec_amount
         |       ,zxr_total_exec_amount
         |       ,final_case_exec_amount
         |       ,final_case_no_exec_amount
         |
         |       ,company_dishonest_info_num_0
         |       ,company_zxr_num_0
         |       ,company_zxr_final_case_num_0
         |       ,company_zxr_restrict_num_0
         |
         |       ,company_dishonest_info_num_1
         |       ,company_zxr_num_1
         |       ,company_zxr_final_case_num_1
         |       ,company_zxr_restrict_num_1
         |
         |       ,deleted
         |       ,get_rowkey_by_prefix(all_rowkey,'company_dishonest_info') as company_dishonest_info_rowkey
         |       ,get_rowkey_by_prefix(all_rowkey,'company_zxr') as company_zxr_rowkey
         |       ,get_rowkey_by_prefix(all_rowkey,'company_zxr_final_case') as company_zxr_final_case_rowkey
         |       ,get_rowkey_by_prefix(all_rowkey,'company_zxr_restrict') as company_zxr_restrict_rowkey
         |from (
         |SELECT  credit_punishment_entity_id
         |        ,collect_set(keyno)[0] AS keyno
         |        ,collect_set(name)[0] AS name
         |        ,collect_set(card_num)[0] AS card_num
         |        ,array_distinct(split(CONCAT_WS(',',collect_set(label)),',')) AS label
         |        ,count(1) as record_num
         |        ,array_distinct(collect_set(split(case_create_time,'-')[0])) AS case_create_time_year
         |        ,double_2_str(sum(CAST(total_exec_amount AS DOUBLE) )) AS total_exec_amount
         |        ,double_2_str(sum(CAST(total_no_exec_amount AS DOUBLE) )) AS total_no_exec_amount
         |        ,double_2_str(sum(CAST(zxr_total_exec_amount AS DOUBLE) )) AS zxr_total_exec_amount
         |        ,double_2_str(sum(CAST(final_case_exec_amount AS DOUBLE) )) AS final_case_exec_amount
         |        ,double_2_str(sum(CAST(final_case_no_exec_amount AS DOUBLE) )) AS final_case_no_exec_amount
         |
         |        ,sum(company_dishonest_info_num_0) as company_dishonest_info_num_0
         |        ,sum(company_zxr_num_0) as company_zxr_num_0
         |        ,sum(company_zxr_final_case_num_0) as company_zxr_final_case_num_0
         |        ,sum(company_zxr_restrict_num_0) as company_zxr_restrict_num_0
         |
         |        ,sum(company_dishonest_info_num_1) as company_dishonest_info_num_1
         |        ,sum(company_zxr_num_1) as company_zxr_num_1
         |        ,sum(company_zxr_final_case_num_1) as company_zxr_final_case_num_1
         |        ,sum(company_zxr_restrict_num_1) as company_zxr_restrict_num_1
         |
         |        ,deleted_merge(deleted) AS deleted
         |        ,array_distinct(split(CONCAT_WS(',',collect_set(all_rowkey)),',')) AS all_rowkey
         |FROM    $org_tab
         |GROUP BY credit_punishment_entity_id
         |)
         |
         |""".stripMargin)
      .cache()
      .createTempView("all_data")


    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $target_tab PARTITION(ds='${ds}')
         |SELECT  t1.credit_punishment_entity_id
         |        ,t1.keyno
         |        ,t1.name
         |        ,0 as entity_type
         |        ,t2.org_number AS card_num
         |        ,t1.label
         |        ,get_area_name_pro(t2.area_code)[0] as province
         |        ,get_area_name_pro(t2.area_code)[1] as city
         |        ,get_area_name_pro(t2.area_code)[2] as county
         |        ,company_other_info(legal_entity_id,legal_entity_name,legal_entity_type,reg_capital,reg_capital_amount,reg_capital_currency,to_millis_timestamp(t2.estiblish_time),logo) as other_info
         |        ,record_num
         |        ,case_create_time_year
         |        ,t1.total_exec_amount
         |        ,t1.total_no_exec_amount
         |        ,t1.zxr_total_exec_amount
         |        ,t1.final_case_exec_amount
         |        ,t1.final_case_no_exec_amount
         |
         |        ,t1.company_dishonest_info_num_0
         |        ,t1.company_zxr_num_0
         |        ,t1.company_zxr_final_case_num_0
         |        ,t1.company_zxr_restrict_num_0
         |
         |        ,t1.company_dishonest_info_num_1
         |        ,t1.company_zxr_num_1
         |        ,t1.company_zxr_final_case_num_1
         |        ,t1.company_zxr_restrict_num_1
         |
         |        ,t1.deleted
         |        ,t1.company_dishonest_info_rowkey
         |        ,t1.company_zxr_rowkey
         |        ,t1.company_zxr_final_case_rowkey
         |        ,t1.company_zxr_restrict_rowkey
         |FROM    (
         |            SELECT  *
         |            FROM    all_data
         |            WHERE   keyno is not null and length(keyno) = 32
         |        ) AS t1
         |LEFT JOIN $all_company AS t2
         |ON      t1.keyno = t2.company_id
         |UNION ALL
         |
         |SELECT  credit_punishment_entity_id
         |        ,keyno
         |        ,name
         |        ,1 as entity_type
         |        ,card_num
         |        ,label
         |        ,get_area_name_pro(SUBSTRING(card_num,0,6))[0] AS province
         |        ,get_area_name_pro(SUBSTRING(card_num,0,6))[1] AS city
         |        ,get_area_name_pro(SUBSTRING(card_num,0,6))[2] AS county
         |        ,person_other_info(get_birth_year(card_num),get_gender(card_num)) as other_info
         |        ,record_num
         |        ,case_create_time_year
         |        ,total_exec_amount
         |        ,total_no_exec_amount
         |        ,zxr_total_exec_amount
         |        ,final_case_exec_amount
         |        ,final_case_no_exec_amount
         |
         |        ,company_dishonest_info_num_0
         |        ,company_zxr_num_0
         |        ,company_zxr_final_case_num_0
         |        ,company_zxr_restrict_num_0
         |
         |        ,company_dishonest_info_num_1
         |        ,company_zxr_num_1
         |        ,company_zxr_final_case_num_1
         |        ,company_zxr_restrict_num_1
         |
         |        ,deleted_person_data(1,name,card_num,deleted) as deleted
         |        ,company_dishonest_info_rowkey
         |        ,company_zxr_rowkey
         |        ,company_zxr_final_case_rowkey
         |        ,company_zxr_restrict_rowkey
         |FROM    all_data
         |WHERE   not (keyno is not null and length(keyno) = 32) and card_num is not null
         |
         |UNION ALL
         |
         |SELECT  credit_punishment_entity_id
         |        ,keyno
         |        ,name
         |        ,1 as entity_type
         |        ,card_num
         |        ,label
         |        ,null AS province
         |        ,null AS city
         |        ,null AS county
         |        ,null as other_info
         |        ,record_num
         |        ,case_create_time_year
         |        ,total_exec_amount
         |        ,total_no_exec_amount
         |        ,zxr_total_exec_amount
         |        ,final_case_exec_amount
         |        ,final_case_no_exec_amount
         |
         |        ,company_dishonest_info_num_0
         |        ,company_zxr_num_0
         |        ,company_zxr_final_case_num_0
         |        ,company_zxr_restrict_num_0
         |
         |        ,company_dishonest_info_num_1
         |        ,company_zxr_num_1
         |        ,company_zxr_final_case_num_1
         |        ,company_zxr_restrict_num_1
         |
         |        ,deleted_person_data(1,name,card_num,deleted) as deleted
         |        ,company_dishonest_info_rowkey
         |        ,company_zxr_rowkey
         |        ,company_zxr_final_case_rowkey
         |        ,company_zxr_restrict_rowkey
         |FROM    all_data
         |WHERE   not (keyno is not null and length(keyno) = 32) and card_num is null
         |
         |""".stripMargin)

  }
}