package com.winhc.bigdata.spark.ng.credit_punishment.udf

import cn.hutool.core.util.NumberUtil
import com.alibaba.fastjson.{JSON, JSONPath}
import com.winhc.bigdata.spark.implicits.BaseHelper._
import com.winhc.bigdata.spark.utils.{BaseUtil, RegCapitalAmount}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

import java.sql.Timestamp
import java.text.DecimalFormat
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import java.util.Locale

/**
 * @author: XuJiakai
 * @date: 2021/5/11 09:23
 */
case class CreditPunishmentCaseAggUDF() extends UserDefinedAggregateFunction {

  private val tn_map = Map(
    "company_dishonest_info" -> "1"
    , "company_zxr" -> "2"
    , "company_zxr_final_case" -> "3"
    , "company_zxr_restrict" -> "4"
  )
  private val tn_name_map = Map(
    "company_dishonest_info" -> "失信被执行人"
    , "company_zxr" -> "被执行人"
    , "company_zxr_final_case" -> "终本案件"
    , "company_zxr_restrict" -> "限制高消费"
  )

  private val delimiter = "@@"


  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("rowkey", StringType)
    , StructField("tn", StringType)
    , StructField("keyno", StringType)
    , StructField("card_num", StringType)
    , StructField("case_create_time", TimestampType)
    , StructField("deleted", LongType)
    , StructField("detail_data", StringType)
  ))

  override def bufferSchema: StructType = StructType(Array(
    StructField("rowkey", ArrayType(StringType, containsNull = false)) // 0
    , StructField("card_num", ArrayType(StringType, containsNull = false)) // 1
    , StructField("keyno", ArrayType(StringType, containsNull = false)) // 2
    , StructField("label", ArrayType(StringType, containsNull = false)) // 3
    , StructField("case_create_time", TimestampType) // 4
    , StructField("total_exec_amount", DoubleType) //累计被执行总金额 5
    , StructField("total_no_exec_amount", DoubleType) //疑似当前欠款总金额 6
    , StructField("zxr_total_exec_amount", DoubleType) //被执行人当前被执行总金额 7
    , StructField("final_case_exec_amount", DoubleType) //终本案件执行标的总金额 8
    , StructField("final_case_no_exec_amount", DoubleType) //终本案件未履行总金额 9
    , StructField("all_rowkey", ArrayType(StringType, containsNull = false)) //所有rowkey的集合，包含历史非历史 10

    , StructField("final_case_exec_info", StringType) //11
    , StructField("dishonest_exec_info", StringType) //12

    , StructField("zxr_exec_amount", DoubleType) //13  被执行人标的额历史+非历史 和5相同
    , StructField("final_case_exec_amount_", DoubleType) //14 终本案件标的额 历史+非历史
    , StructField("deleted", ArrayType(LongType, containsNull = false)) //15 deleted状态
  ))

  override def dataType: DataType = DataTypes.createMapType(StringType, StringType)

  override def deterministic: Boolean = false

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Seq.empty[String])
    buffer.update(1, Seq.empty[String])
    buffer.update(2, Seq.empty[String])
    buffer.update(3, Seq.empty[String])
    buffer.update(4, null)
    buffer.update(5, 0d)
    buffer.update(6, 0d)
    buffer.update(7, 0d)
    buffer.update(8, 0d)
    buffer.update(9, 0d)
    buffer.update(10, Seq.empty[String])

    buffer.update(11, null)
    buffer.update(12, null)

    buffer.update(13, 0d)
    buffer.update(14, 0d)
    buffer.update(15, Seq.empty[Long])
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    val rowkey = input.getString(0)
    val tn = input.getString(1)
    val keyno = input.getString(2)
    val card_num = input.getString(3)
    val case_create_time = input.getTimestamp(4)
    val deleted = input.getLong(5)
    val detail_data = input.getString(6)

    buffer(10) = s"$tn$delimiter$deleted$delimiter$rowkey" +: buffer.getSeq[String](10)
    buffer(2) = keyno +: buffer.getSeq[String](2)
    buffer(1) = card_num +: buffer.getSeq[String](1)
    buffer(15) = deleted +: buffer.getSeq[Long](15)
    buffer(4) = getCaseCreateTime(buffer.getTimestamp(4), case_create_time)

    deleted match {
      //不计历史信息：
      case 0 => {
        buffer(0) = s"$tn$delimiter$rowkey" +: buffer.getSeq[String](0)
        buffer(3) = tn +: buffer.getSeq[String](3)

        if (tn.equals("company_zxr")) {
          buffer(6) = buffer.getDouble(6) + getAmount(detail_data, "$.exec_money")
        }

        if (tn.equals("company_zxr_final_case")) {
          val exec_money = getAmount(detail_data, "$.exec_amount")
          val no_exec_amount = getAmount(detail_data, "$.no_exec_amount")
          if (exec_money != 0 && no_exec_amount != 0) {
            var str: String = null
            if (no_exec_amount > exec_money) {
              str = "履行0%"
            } else {
              str = NumberUtil.decimalFormat("履行#.##%", NumberUtil.sub(1, NumberUtil.div(no_exec_amount, exec_money)))
            }
            buffer(11) = str
          }

          buffer(6) = buffer.getDouble(6) + getAmount(detail_data, "$.no_exec_amount")

          buffer(8) = buffer.getDouble(8) + getAmount(detail_data, "$.exec_amount")
          buffer(9) = buffer.getDouble(9) + getAmount(detail_data, "$.no_exec_amount")

        }
        if (tn.equals("company_dishonest_info")) {
          buffer(12) = getStr(detail_data, "$.performance")
        }

        if (tn.equals("company_zxr")) {
          buffer(7) = buffer.getDouble(7) + getAmount(detail_data, "$.exec_money")
        }
      }
      case 1 => {
      }
      case _ => return
    }


    tn match {
      case "company_zxr" => {
        buffer(5) = buffer.getDouble(5) + getAmount(detail_data, "$.exec_money")
        buffer(13) = buffer.getDouble(13) + getAmount(detail_data, "$.exec_money")
      }
      case "company_zxr_final_case" => {
        buffer(14) = buffer.getDouble(14) + getAmount(detail_data, "$.exec_amount")
      }
      case _ => {}
    }

  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    buffer1(0) = buffer1.getSeq[String](0) ++ buffer2.getSeq[String](0)
    buffer1(1) = buffer1.getSeq[String](1) ++ buffer2.getSeq[String](1)
    buffer1(2) = buffer1.getSeq[String](2) ++ buffer2.getSeq[String](2)
    buffer1(3) = buffer1.getSeq[String](3) ++ buffer2.getSeq[String](3)
    buffer1(4) = getCaseCreateTime(buffer1.getTimestamp(4), buffer2.getTimestamp(4))
    buffer1(5) = buffer1.getDouble(5) + buffer2.getDouble(5)
    buffer1(6) = buffer1.getDouble(6) + buffer2.getDouble(6)
    buffer1(7) = buffer1.getDouble(7) + buffer2.getDouble(7)
    buffer1(8) = buffer1.getDouble(8) + buffer2.getDouble(8)
    buffer1(9) = buffer1.getDouble(9) + buffer2.getDouble(9)
    buffer1(10) = buffer1.getSeq[String](10) ++ buffer2.getSeq[String](10)

    if (buffer1.getString(11) == null) {
      buffer1(11) = buffer2.getString(11)
    }
    if (buffer1.getString(12) == null) {
      buffer1(12) = buffer2.getString(12)
    }

    buffer1(13) = buffer1.getDouble(13) + buffer2.getDouble(13)
    buffer1(14) = buffer1.getDouble(14) + buffer2.getDouble(14)
    buffer1(15) = buffer1.getSeq[Long](15) ++ buffer2.getSeq[Long](15)
  }

  override def evaluate(buffer: Row): Any = {
    val strings = buffer.getSeq[String](0).distinct.filter(StringUtils.isNotBlank)
    val all_rowkey = buffer.getSeq[String](10).distinct.filter(StringUtils.isNotBlank)


    val stringToInt: Map[String, Int] = all_rowkey.map(r => {
      val ss = r.split(delimiter)
      (ss(0), ss(1))
    }).filter(r => r._2.equals("0") || r._2.equals("1")).map(r => s"${r._1}_${r._2}").groupBy(f => f).mapValues(_.size)

    val rowkey = strings
      .map(r => {
        val s = r.split(delimiter)
        s"${tn_map(s(0))}$delimiter${s(1)}"
      })
    val card_num: Seq[String] = buffer.getSeq[String](1).distinct
      .filter(BaseUtil.is_id_card(_)).map(BaseUtil.id_card_trim).filter(StringUtils.isNotBlank)

    val keyno = buffer.getSeq[String](2).distinct.filter(StringUtils.isNotBlank)
    val label = buffer.getSeq[String](3).distinct.map(tn_name_map(_))
    val case_create_time = buffer.getTimestamp(4)
    val total_exec_amount = buffer.getDouble(5)
    val total_no_exec_amount = buffer.getDouble(6)
    val zxr_total_exec_amount = buffer.getDouble(7)
    val final_case_exec_amount = buffer.getDouble(8)
    val final_case_no_exec_amount = buffer.getDouble(9)
    val df = DateTimeFormatter.ofPattern("yyyy-MM-dd").withLocale(Locale.CHINA).withZone(ZoneId.systemDefault)

    val cct = if (case_create_time == null) null else df.format(case_create_time.toInstant)

    val del = buffer.getSeq[Long](15).distinct
    val deleted = if (del.contains(0)) "0" else if (del.contains(1)) "1" else "9"

    //    val deleted = if (rowkey.isEmpty) "1" else "0"

    val e1 = buffer.getString(11)
    val e2 = buffer.getString(12)

    val exec_info = if (e1 == null) if (e2 == null) "未知" else e2 else e1

    val a1 = getAmountInvolved(buffer.getDouble(13))
    val a2 = getAmountInvolved(buffer.getDouble(14))

    val amount_involved = if (a1 == null) if (a2 == null) "未知" else a2 else a1

    Map(
      "rowkey" -> rowkey.mkString(",")
      , "card_num" -> card_num.headOption.getOrElse(null)
      , "keyno" -> keyno.headOption.getOrElse(null)
      , "label" -> label.mkStringOrNull()
      , "case_create_time" -> cct
      , "record_num" -> s"${rowkey.length}"

      , "exec_info" -> exec_info
      , "amount_involved" -> amount_involved

      , "total_exec_amount" -> double2String(total_exec_amount)
      , "total_no_exec_amount" -> double2String(total_no_exec_amount)
      , "zxr_total_exec_amount" -> double2String(zxr_total_exec_amount)
      , "final_case_exec_amount" -> double2String(final_case_exec_amount)
      , "final_case_no_exec_amount" -> double2String(final_case_no_exec_amount)

      , "company_dishonest_info_num_0" -> s"${stringToInt.getOrElse("company_dishonest_info_0", 0)}"
      , "company_zxr_num_0" -> s"${stringToInt.getOrElse("company_zxr_0", 0)}"
      , "company_zxr_final_case_num_0" -> s"${stringToInt.getOrElse("company_zxr_final_case_0", 0)}"
      , "company_zxr_restrict_num_0" -> s"${stringToInt.getOrElse("company_zxr_restrict_0", 0)}"

      , "company_dishonest_info_num_1" -> s"${stringToInt.getOrElse("company_dishonest_info_1", 0)}"
      , "company_zxr_num_1" -> s"${stringToInt.getOrElse("company_zxr_1", 0)}"
      , "company_zxr_final_case_num_1" -> s"${stringToInt.getOrElse("company_zxr_final_case_1", 0)}"
      , "company_zxr_restrict_num_1" -> s"${stringToInt.getOrElse("company_zxr_restrict_1", 0)}"

      , "deleted" -> deleted
      , "all_rowkey" -> all_rowkey.mkString(",")
    )

  }

  private def getAmountInvolved(d: Double): String = if (d == 0d) null else NumberUtil.decimalFormat("#.##万元", NumberUtil.div(d, 10000))


  private def getStr(json: String, jsonPath: String): String = {
    JSONPath.eval(JSON.parseObject(json), jsonPath).asInstanceOf[String]
  }

  private def getAmount(json: String, jsonPath: String): Double = {
    val str = getStr(json, jsonPath)
    try {
      RegCapitalAmount.getAmount(str).toDouble / 100
    } catch {
      case ex: Exception => 0d
    }
  }


  private def getCaseCreateTime(time1: Timestamp, time2: Timestamp): Timestamp = {
    if (time2 == null && time1 == null)
      return null
    if (time2 == null)
      return time1
    if (time1 == null)
      return time2

    if (time2.compareTo(time1) > 0)
      time1
    else
      time2
  }

  private val dof = new DecimalFormat("0.##")

  private def double2String(d: Double): String = dof.format(d)
}
