package com.winhc.bigdata.spark.ng.dynamic

import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/6/22 09:13
 */
case class NgCompanyDynamic(s: SparkSession,
                            args: Seq[NgCompanyDynamicArgs]
                            , agg: Seq[NgAcrossTabAggArgs]
                            , inc: Boolean = true
                           ) extends LoggingUtils with Logging {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = if (inc) "winhc_ng.out_company_dynamic" else "winhc_ng.out_company_dynamic_all"

  private val args_map: Map[String, NgCompanyDynamicArgs] = args.map(r => (r.tn, r)).toMap

  init()

  def init(): Unit = {
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    id STRING COMMENT '企业动态主键'
         |    ,association_entity_info STRING COMMENT '关联主体信息,json格式。[{keyno:xxx,name:xxx}]'
         |    ,rowkey STRING COMMENT '企业动态涉及到的rowkey,如果是聚合类型的数据此项为空'
         |    ,tn STRING COMMENT 'rowkey字段对应的tn,如果是聚合类型的数据此项可能为空'
         |    ,update_type STRING COMMENT '动态类型：insert，update，deleted，remove'
         |    ,risk_level STRING COMMENT '变更风险等级'
         |    ,risk_level_detail STRING COMMENT '变更风险等级'
         |    ,dynamic_code STRING COMMENT 'dynamic_info code'
         |    ,dynamic_info STRING COMMENT '动态展示层的相关数据，json格式'
         |    ,agg_detail_text STRING COMMENT '聚合类型的json数据，规范优先'
         |    ,agg_detail_rowkey STRING COMMENT '聚合类型rowkey的多项rowkey字段，结构：tn@@rowkey,tn@@rowkey'
         |    ,collapse_key STRING COMMENT '用于折叠的字段，concat_ws("_",tn,dynamic_time)。跟据动态时间来折叠'
         |    ,biz_time string COMMENT '业务上的变更时间（业务展示 yyyy-MM-dd）'
         |    ,dynamic_time STRING  COMMENT  '动态时间，用于业务展示（yyyy-MM-dd HH:mm:ss 存量数据为业务变更时间，增量数据为当前动态新建时间）'
         |    ,update_time STRING COMMENT '更新时间'
         |    ,create_time STRING COMMENT '创建时间'
         |    ,deleted BIGINT  COMMENT '非0删除'
         |)
         |COMMENT '企业动态输出表'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)

    val i = agg.flatMap(_.tabs).map((_, 1)).groupBy(_._1)
      .mapValues(_.foldLeft(0)(_ + _._2)).count(_._2 >= 2)
    if (i > 0) {
      throw new RuntimeException("agg.tn 有交叉")
    }
  }

  private lazy val org_tab = if (inc) "winhc_ng.bds_change_extract" else "winhc_ng.bds_change_extract_all"
  private lazy val ds = getLastPartitionsOrElse(org_tab, "0")

  private def get_rdd(tn: String): RDD[CompanyDynamicRecord] = {
    val rdd = sql(
      s"""
         |SELECT  *
         |FROM    $org_tab
         |WHERE   ds = $ds
         |AND     tn = '$tn'
         |""".stripMargin)
      .rdd.map(r => {
      val value = r.getAs[String]("change_fields")
      val change_fields: Seq[String] = if (StringUtils.isEmpty(value)) Seq.empty else value.split(",")
      ChangeExtract(rowkey = r.getAs("rowkey")
        , company_id = r.getAs("company_id")
        , company_name = null
        , tn = r.getAs("table_name")
        , update_type = r.getAs("update_type")
        , old_data = r.getAs("old_data")
        , new_data = r.getAs("new_data")
        , change_fields = change_fields
        , biz_date = r.getAs("biz_date")
        , update_time = r.getAs("update_time")
      )
    }).filter(r => {
      val filter = args_map(r.tn).filter
      if (filter == null) {
        true
        //        CompanyDynamicUtils.default_filter(r.update_type, r.biz_date, r.change_fields, r.old_data, r.new_data)
      } else {
        filter.apply(r.update_type, r.biz_date, r.change_fields, r.old_data, r.new_data)
      }
    }).flatMap(r => args_map(r.tn).flat_map.apply(r))
      .map(_.format())
      .filter(_ != null)

    rdd
  }

  def calc(): Unit = {
    //todo  可将rdd直接落hbase库

    var rdd_map: mutable.Map[String, RDD[CompanyDynamicRecord]] = mutable.Map.empty

    for (elem <- args) {
      var tmp_rdd: RDD[CompanyDynamicRecord] = get_rdd(elem.tn)

      if (elem.group_by_key != null) {
        if (elem.group_by_pre == null) {
          println(s"elem.tn = ${elem.tn} : groupBy and flatMap !")
          tmp_rdd = tmp_rdd
            .groupBy(r => args_map(elem.tn).group_by_key.apply(r))
            .flatMap(r => args_map(elem.tn).group_by_flat_map(r._2.toSeq))
        } else {
          println(s"elem.tn = ${elem.tn} : flatMap , groupBy and flatMap")
          tmp_rdd = tmp_rdd
            .flatMap(r => args_map(elem.tn).group_by_pre.apply(r))
            .filter(_ != null)
            .groupBy(r => args_map(elem.tn).group_by_key.apply(r))
            .flatMap(r => args_map(elem.tn).group_by_flat_map(r._2.toSeq))
        }
      }
      rdd_map = rdd_map + (elem.tn -> tmp_rdd)
    }

    for (elem <- agg) {
      var tmp_rdd: RDD[CompanyDynamicRecord] = null
      for (tn <- elem.tabs) {
        if (tmp_rdd == null) {
          tmp_rdd = rdd_map(tn)
        } else {
          tmp_rdd = tmp_rdd.union(rdd_map(tn))
        }
        rdd_map = rdd_map - tn
      }
      if (elem.group_by_pre == null) {
        println(s"agg : elem.tabs = ${elem.tabs} : groupBy and flatMap")
        tmp_rdd = tmp_rdd.groupBy(elem.group_by_key).flatMap(r => elem.group_by_flat_map.apply(r._2.toSeq))
      } else {
        println(s"agg : elem.tabs = ${elem.tabs} : flatMap , groupBy and flatMap")
        tmp_rdd = tmp_rdd
          .flatMap(r => elem.group_by_pre.apply(r))
          .groupBy(elem.group_by_key).flatMap(r => elem.group_by_flat_map.apply(r._2.toSeq))
      }
      rdd_map = rdd_map + (elem.tabs.mkString("_") -> tmp_rdd)
    }

    var out_rdd: RDD[CompanyDynamicRecord] = null

    for ((key, value) <- rdd_map) {
      println("union key: " + key)
      if (out_rdd == null) {
        out_rdd = value
      } else {
        out_rdd = out_rdd.union(value)
      }
    }

    spark.createDataFrame(out_rdd.map(_.to_row()), spark.table(target_tab).schema)
      .createTempView("company_dynamic_out_tab")
    val cols = getColumns(target_tab).diff(Seq("ds")).mkString(",")

    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"}  TABLE $target_tab PARTITION(ds='${BaseUtil.getYesterday()}')
         |SELECT $cols
         |FROM   company_dynamic_out_tab
         |""".stripMargin)
  }
}

object NgCompanyDynamic {
  def main(args: Array[String]): Unit = {
    var flag: String = null
    if (args.size == 1) {
      flag = args(0)
    } else {
      flag = "inc"
    }
    var inc = true
    if (!"inc".equals(flag)) {
      inc = false
    }

    println(s"${if (inc) "inc..." else "all..."}")

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    //    val tn = "auction_tracking,company_land_announcement,company_own_tax"
    //    val tn = "company_holder"
    val tn = "all"
    var dynamicArgs = NgCompanyDynamicArgs.getStartArgs(inc)
    var aggArgs = NgCompanyDynamicArgs.getAggArgs
    if (!"all".equals(tn)) {
      val set = tn.split(",").toSet
      dynamicArgs = dynamicArgs.filter(r => set.contains(r.tn))
      aggArgs = aggArgs.filter(r => {
        r.tabs.forall(s => set.contains(s))
      })
    }
    NgCompanyDynamic(spark, dynamicArgs, aggArgs, inc = inc).calc()
    spark.stop()
  }
}
