package com.winhc.bigdata.spark.ng.pull

import cn.hutool.core.codec.Base64
import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.{LoggingUtils, SparkUtils}
import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions.{col, struct, to_json}
import scalaj.http.{Http, HttpOptions}

import scala.annotation.meta.getter
import scala.collection.{JavaConverters, mutable}
import scala.util.Random

/**
 * @author: XuJiakai
 * @date: 2021/4/13 09:57
 */
case class PullDataJob(s: SparkSession
                       , task_id: String
                      ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = "winhc_ng.eci_distribution_data"

  private val detail_data = Seq("credit_code", "reg_number", "estiblish_time", "cate_first_code", "cate_second_code", "cate_third_code","province_code","city_code","county_code","new_cid")

  init()

  private def init(): Unit = {
    json_2_array_udf()

    def generate_random(n: Int): Int = {
      val random = new Random()
      random.nextInt(n)
    }

    spark.udf.register("generate_random", generate_random _)

    def get_company_name(name: String): String = {
      try {
        JSON.parseObject(name).getString("show")
      } catch {
        case ex: Exception => {}
          ""
      }
    }

    spark.udf.register("get_company_name", get_company_name _)

    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    company_id  STRING COMMENT 'company id'
         |    ,company_name STRING COMMENT 'company name'
         |    ,detail_data  String COMMENT '其它数据, json格式'
         |    ,export_time TIMESTAMP COMMENT '导出时间'
         |)
         |COMMENT '线上数据分发表'
         |PARTITIONED BY (
         |    dt STRING COMMENT '分区'
         |)
         |""".stripMargin)
  }


  private def get_v8_data(): Unit = {
    val d = detail_data.diff(Seq("company_org_type_std", "company_score_weight")).mkString(",")
    val tmp = "$.id"
    sql(
      s"""
         |SELECT  id AS company_id
         |        ,get_company_name(cname) AS company_name
         |        ,company_org_type_std
         |        ,company_score_weight
         |        ,$d
         |        ,size(json_2_array(holder,'$tmp')) AS holder_size
         |        ,size(json_2_array(staff,'$tmp')) AS staff_size
         |        ,size(split(company_tm,',')) AS tm_size
         |        ,size(split(icp,',')) AS icp_size
         |        ,size(split(app_info,',')) AS app_size
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY id ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  *
         |                        FROM    winhc_ng.out_company_v8_index
         |                        WHERE   ds > 0
         |                    )
         |        )
         |WHERE   num = 1
         |""".stripMargin)
      .withColumn("detail_data", to_json(struct(detail_data.map(col): _*)))
      .cache()
      .createTempView("all_company_data")
  }


  private def insert_tab(company_score_weight_gte: String, company_score_weight_lt: String, task_id: String, rank: Int, sub_dt_size: Int): Unit = {
    var seq: Seq[String] = Seq()
    if (company_score_weight_gte != null) {
      seq = seq :+ s" company_score_weight >= $company_score_weight_gte "
    }
    if (company_score_weight_lt != null) {
      seq = seq :+ s" company_score_weight < $company_score_weight_lt "
    }
    val where = seq.mkString("AND")

    if (sub_dt_size <= 1)
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_tab PARTITION(dt='${task_id}-${rank}-0')
           |SELECT company_id
           |       ,company_name
           |       ,detail_data
           |       ,now() as export_time
           |FROM   all_company_data
           |WHERE  $where
           |""".stripMargin)
    else
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_tab PARTITION(dt)
           |SELECT company_id
           |       ,company_name
           |       ,detail_data
           |       ,now() as export_time
           |       ,concat('${task_id}-${rank}-',generate_random($sub_dt_size)) as dt
           |FROM   all_company_data
           |WHERE  $where
           |""".stripMargin)
  }

  def dispatch(dataDistribution: String): Unit = {
    get_v8_data()
    var vo: Seq[TablePartitionInfoVO] = Seq.empty
    val infos = get_rank(dataDistribution)
    for (elem <- infos) {
      insert_tab(company_score_weight_gte = elem.gte, company_score_weight_lt = elem.lt, task_id = task_id, rank = elem.rank, sub_dt_size = elem.sub_size)
      if (elem.sub_size <= 1) {
        vo = vo :+ TablePartitionInfoVO(taskId = task_id, maxComputeTableName = target_tab, partitionName = s"${task_id}-${elem.rank}-0", recordCount = 0, priorityLevel = elem.rank)
      } else {
        for (i <- 0 until elem.sub_size) {
          vo = vo :+ TablePartitionInfoVO(taskId = task_id, maxComputeTableName = target_tab, partitionName = s"${task_id}-${elem.rank}-$i", recordCount = 0, priorityLevel = elem.rank)
        }
      }
    }

    val map = get_partition_size(vo.map(e => e.partitionName))
    vo = vo.map(e => {
      TablePartitionInfoVO(taskId = e.taskId, maxComputeTableName = e.maxComputeTableName, partitionName = e.partitionName, recordCount = map.getOrElse(e.partitionName, -1), priorityLevel = e.priorityLevel)
    })

    call_back(vo)
  }


  private def call_back(vo: Seq[TablePartitionInfoVO]): Unit = {
    try {
      println("call back! ")
      val result = Http("http://192.168.2.54:8588/call-back?sign=hnjmkladfsvhjklnmadsvfhjklmnasdfvgbkln")
        .postData(vo.toJson())
        .header("Content-Type", "application/json")
        .header("Charset", "UTF-8")
        .option(HttpOptions.readTimeout(10000)).asString
      println(result)
    } catch {
      case ex: Exception => {
        ex.printStackTrace()
      }
    }
  }


  private def testCallBack(): Unit = {
    val body =
      """
        |[{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-0-0","record_count":16338440,"priorityLevel":0},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-0","record_count":10589879,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-1","record_count":10595066,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-2","record_count":10594918,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-3","record_count":10593033,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-4","record_count":10591287,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-5","record_count":10592954,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-6","record_count":10598344,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-7","record_count":10594058,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-1-8","record_count":10589020,"priorityLevel":1},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-2-0","record_count":371513,"priorityLevel":2},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-0","record_count":10222473,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-1","record_count":10223483,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-2","record_count":10222247,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-3","record_count":10221756,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-4","record_count":10221484,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-5","record_count":10220512,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-6","record_count":10223675,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-7","record_count":10226255,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-3-8","record_count":10219486,"priorityLevel":3},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-4-0","record_count":10317257,"priorityLevel":4},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-5-0","record_count":11574350,"priorityLevel":5},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-6-0","record_count":11508539,"priorityLevel":6},{"taskId":"20210413_asdfg4","maxComputeTableName":"winhc_ng.eci_distribution_data","partition_name":"20210413_asdfg4-7-0","record_count":7895370,"priorityLevel":7}]
        |""".stripMargin
    try {
      println("call back! ")
      val result = Http("http://192.168.2.54:8588/call-back")
        .postData(body)
        .header("Content-Type", "application/json")
        .header("Charset", "UTF-8")
        .option(HttpOptions.readTimeout(10000)).asString
      println(result)
    } catch {
      case ex: Exception => {
        ex.printStackTrace()

      }
    }
  }

  private def get_partition_size(ps: Seq[String]): Map[String, Long] = {
    sql(
      s"""
         |SELECT  dt
         |        ,COUNT(1) as c
         |FROM    $target_tab
         |WHERE   dt IN ${ps.map(e => s"'$e'").mkString("(", ",", ")")}
         |GROUP BY dt
         |""".stripMargin)
      .collect().map(r => {
      (r.getString(0), r.getLong(1))
    }).toMap
  }

  private def get_rank(dataDistribution: String): Seq[rankInfo] = {

    def add_gte(v: String): String = {
      if (v == null)
        return null
      if (v.toInt == 0)
        return null
      s"${v.toInt - 1}"
    }

    def add_sub_size(i: Int): Int = if (i <= 0) return 1 else i

    val obj = JSON.parseObject(dataDistribution)
    val seq = JavaConverters.asScalaIteratorConverter(obj.keySet().iterator()).asScala.toSeq
    val code_data: mutable.SortedSet[data_distribution] = mutable.SortedSet()
    for (e <- seq) {
      code_data.add(data_distribution(rank = e, count = obj.getLongValue(e)))
    }
    var list: mutable.Seq[rankInfo] = mutable.Seq()
    var count: Long = 0
    var gte: String = null
    var rank = 0

    for (e <- code_data) {
      if (gte == null) {
        gte = e.rank
      }
      count = count + e.count
      if (count > 10000000 && count - e.count != 0) {
        list = list :+ rankInfo(gte = add_gte(gte), lt = e.rank, num = count - e.count, sub_size = add_sub_size(((count - e.count) / 10000000).toInt), rank = rank)
        gte = null
        count = e.count
        rank = rank + 1
      }
    }
    list = list :+ rankInfo(gte = add_gte(gte), lt = null, num = count, sub_size = add_sub_size(((count) / 10000000).toInt), rank = rank)

    list.toSeq

    /*
        Seq(
          rankInfo(gte = null, lt = "1", num = 16338440, sub_size = 1, rank = 0)
          , rankInfo(gte = "1", lt = "2", num = 95338559, sub_size = 9, rank = 1)
          , rankInfo(gte = "2", lt = "4", num = 371513, sub_size = 1, rank = 2)
          , rankInfo(gte = "4", lt = "5", num = 92001371, sub_size = 9, rank = 3)
          , rankInfo(gte = "5", lt = "6", num = 10317257, sub_size = 1, rank = 4)
          , rankInfo(gte = "6", lt = "7", num = 11574350, sub_size = 1, rank = 5)
          , rankInfo(gte = "7", lt = "8", num = 11508539, sub_size = 1, rank = 6)
          , rankInfo(gte = "8", lt = null, num = 8508539, sub_size = 1, rank = 7)
        )*/
  }

  case class data_distribution(rank: String, count: Long) extends Comparable[data_distribution] {
    override def hashCode(): Int = s"$rank$count".hashCode()

    override def equals(obj: Any): Boolean =
      obj match {
        case d: data_distribution =>
          s"$rank$count".equals(s"${d.rank}${d.count}")
        case _ =>
          false
      }

    override def compareTo(o: data_distribution): Int = rank.toInt.compareTo(o.rank.toInt)
  }

  case class rankInfo(gte: String, lt: String, num: Long, sub_size: Int, rank: Int)

  case class TablePartitionInfoVO(taskId: String, maxComputeTableName: String, partitionName: String, recordCount: Long, priorityLevel: Int)

}


object PullDataJob {
  def main(args: Array[String]): Unit = {
//    val dataDistribution = "{\"0\":1633844,\"1\":1953385,\"2\":235649,\"3\":135864,\"4\":92001371,\"5\":10317257,\"6\":11574350,\"7\":11508539,\"8\":4893124,\"9\":1615269,\"10\":1386977}"

    val Array(task_id, dataDistribution) = args
    val project = "winhc_ng"
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    PullDataJob(spark, task_id)
      .dispatch(Base64.decodeStr(dataDistribution))
    spark.stop()

  }
}
