package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions.{col, struct, to_json}

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/4/28 17:35
 */
case class WinhcNgSummary_new(s: SparkSession,
                              project: String //表所在工程名
                              , target_tab: String
                              , args: Seq[SummaryArgs]
                             ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s
  init()
  private lazy val ds = getLastPartitionsOrElse(target_tab, null)


  private def init() {
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    company_id  STRING COMMENT '公司或人的id'
         |    ,summary STRING COMMENT '格子中的摘要信息，json格式'
         |    ,detail STRING COMMENT '个别维度详细的摘要信息'
         |)
         |COMMENT 'out es summary,create by ${BaseUtil.nowDate(pattern = "yyyy-MM-dd HH:mm:ss")}'
         |PARTITIONED BY (
         |  `ds` STRING COMMENT '时间分区',
         |  `tn` STRING COMMENT 'tn分区'
         |  )
         |LIFECYCLE 15
         |""".stripMargin)
  }


  private def get_table_data(arg: SummaryArgs): String = {

    val tab = arg.table_name
    val companyIdField = arg.companyIdField
    val distinctField = arg.distinctField

    val sortField = arg.sortField
    val where = if (StringUtils.isEmpty(arg.where)) {
      s""
    } else {
      s"AND   ${arg.where}"
    }


    val ads_table = s"${project}.ads_$tab" //存量ads表
    val inc_ads_table = s"${project}.inc_ads_$tab"

    val inc_ads_last_ds = getLastPartitionsOrElse(inc_ads_table, "0")

    val new_cols = getColumns(ads_table).intersect(getColumns(inc_ads_table))

    val ads_last_ds = getLastPartitionsOrElse(ads_table, "0")
    val tab_tmp_view = s"${arg.winhc_hash()}_data"

    if (ds == null) {
      sql(
        s"""
           |SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
           |FROM    $ads_table
           |WHERE   ds = '$ads_last_ds'
           |UNION ALL
           |SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
           |FROM    $inc_ads_table
           |WHERE   ds > $ads_last_ds
           |""".stripMargin)
        .createOrReplaceTempView(tab_tmp_view)
    } else {
      sql(
        s"""
           |SELECT  ${new_cols.map(getCastCols(_, "org_tab.")).mkString(",")}
           |FROM    (
           |            SELECT  DISTINCT $companyIdField as xjk_cid
           |            FROM    $inc_ads_table
           |            WHERE   ds = $inc_ads_last_ds
           |        ) id_table
           |JOIN (
           |              SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
           |                      ,$companyIdField as xjk_cid
           |              FROM    $inc_ads_table
           |              WHERE   ds > '$ads_last_ds'
           |              AND     ds < '$inc_ads_last_ds'
           |              UNION ALL
           |              SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
           |                      ,$companyIdField as xjk_cid
           |              FROM    $ads_table
           |              WHERE   ds = '$ads_last_ds'
           |          ) org_tab
           |ON      id_table.xjk_cid = org_tab.xjk_cid
           |UNION ALL
           |SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
           |FROM    $inc_ads_table
           |WHERE   ds = $inc_ads_last_ds
           |""".stripMargin)
        .createOrReplaceTempView(tab_tmp_view)
    }

    val distinct_tab = s"${tab_tmp_view}_distinct"
    sql(
      s"""
         |SELECT  ${new_cols.map(getCastCols(_, "")).mkString(",")}
         |FROM    (
         |            SELECT  tmp.*
         |                    ,ROW_NUMBER() OVER(PARTITION BY $distinctField ORDER BY $sortField DESC ) c
         |            FROM    $tab_tmp_view AS tmp
         |        ) tmp2
         |WHERE   tmp2.c = 1
         |$where
         |""".stripMargin)
      .createOrReplaceTempView(distinct_tab)
    distinct_tab
  }

  private def get_tab_summary(arg: SummaryArgs): String = {
    val tab_hash = arg.winhc_hash()

    val tab = arg.table_name
    val companyIdField = arg.companyIdField

    val result_tab = s"${tab_hash}_summary_tab"

    val all_tab = get_table_data(arg)

    val func_name = s"xjk_func_${tab_hash}"

    val view = if (arg.groupByInfo == null) {
      if (arg.field_prefix != null) {
        s"arr[0] as ${arg.field_prefix}"
      } else {
        s"arr[0] as $tab"
      }

    } else {
      arg.groupByInfo.value_alias.indices.map(i => {
        s"arr[$i] as ${arg.groupByInfo.value_alias(i)._2}"
      }).mkString(",")
    }

    //注册函数
    if (arg.groupByInfo != null) {
      val fieldSeq = arg.groupByInfo.value_alias.map(r => {
        (s"${r._1}", r._2)
      })

      def getResArr(group_val: String, num: Long): Seq[Long] = {
        val res = scala.collection.mutable.ArrayBuffer[Long]()
        for (i <- fieldSeq) {
          if (i._1.equals(group_val)) {
            res += num
          } else {
            res += 0
          }
        }
        res
      }

      spark.udf.register(func_name, getResArr _)
    }

    val groupKey_show = if (arg.groupByInfo == null) {
      s",array(count(1)) as arr"
    } else {
      s",$func_name(cast(${arg.groupByInfo.field} as STRING),count(1)) as arr"
    }

    val groupKey = if (arg.groupByInfo == null) {
      s""
    } else {
      s",${arg.groupByInfo.field}"
    }

    sql(
      s"""
         |SELECT  company_id
         |        ,${view}
         |FROM    (
         |        SELECT  $companyIdField as company_id
         |                $groupKey_show
         |        FROM    $all_tab
         |        GROUP BY $companyIdField ${groupKey}
         |)
         |""".stripMargin)
      .createOrReplaceTempView(result_tab)

    if (arg.groupByInfo != null) {
      sql(
        s"""
           |SELECT  company_id
           |        ,${arg.groupByInfo.value_alias.map(_._2).map(f => s"sum($f) as $f").mkString(",")}
           |FROM    $result_tab
           |GROUP BY company_id
           |""".stripMargin)
        .createOrReplaceTempView(result_tab)
    }
    result_tab
  }


  def calc(): Unit = {
    val summary_tabs = args.map(get_tab_summary).seq

    for (elem <- summary_tabs) {
      val cols = getColumns(elem).diff(Seq("company_id"))
      sql(
        s"""
           |select * from $elem
           |""".stripMargin)
        .withColumn("summary", to_json(struct(cols.map(col): _*)))
        .createTempView(s"xjk_tmp_summary_tab_$elem")
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_tab PARTITION(ds='${BaseUtil.getYesterday()}',tn='$elem')
           |SELECT company_id,summary,null as detail
           |FROM
           |    xjk_tmp_summary_tab_$elem
           |""".stripMargin)
    }


    /* val merge = merge_table(spark, summary_tabs, "company_id")

     merge.calc(summary_tab)
     val cols = getColumns(summary_tab).diff(Seq("company_id"))

     sql(
       s"""
          |select * from $summary_tab
          |""".stripMargin)
       .withColumn("summary", to_json(struct(cols.map(col): _*)))
       .createTempView("xjk_tmp_summary_tab")

     sql(
       s"""
          |INSERT OVERWRITE TABLE $target_tab PARTITION(ds='${BaseUtil.getYesterday()}')
          |SELECT company_id,summary,null as detail
          |FROM
          |    xjk_tmp_summary_tab
          |""".stripMargin)
     merge.drop()*/
  }

  private def getCastCols(name: String, pre: String): String = {
    val list = List("cid", "new_cid", "ncid")
    if (list.contains(name)) {
      return s"CAST(${pre}${name} as BIGINT) $name"
    }
    pre + name
  }
}