package com.winhc.bigdata.spark.utils

import java.util.concurrent.{CountDownLatch, TimeUnit}
import org.apache.commons.lang3.time.StopWatch
import org.apache.spark.JobExecutionStatus
import org.apache.spark.internal.Logging
import org.apache.spark.sql.SparkSession

import java.util.concurrent.atomic.AtomicInteger

/**
 * @Author: XuJiakai
 * @Date: 2020/8/21 11:36
 * @Description:
 */
object AsyncExtract {

  def startAndWait(spark: SparkSession, seq: Seq[(String, () => Boolean)]): Unit = {
    start(seq)
    wait(spark)
  }

  def start(seq: Seq[(String, () => Boolean)]): Unit = {
    AsyncExtract(seq).start()
  }

  def wait(spark: SparkSession): Unit = {
    val tracker = spark.sparkContext.statusTracker

    val ints = tracker.getJobIdsForGroup(null)
    println(ints.mkString(","))
    val failed = ints
      .map(i => tracker.getJobInfo(i))
      .map(o => o.orNull)
      .filter(_ != null)
      .map(i => i.status()).exists(i => JobExecutionStatus.FAILED.equals(i))
    if (failed) {
      println("There are failed jobs !!!")
      sys.exit(-999)
    }
  }
}

case class AsyncExtract(seq: Seq[(String, () => Boolean)]) extends Logging {

  def start(): Unit = {
    val latch = new CountDownLatch(seq.length)
    val error_count = new AtomicInteger(0)
    seq.foreach(e => {
      asyncWatch(e._1, () => {
        try {
          println("______________________________" + e._1 + "___________________________________")
          val res = e._2()
          println(s"---${e._1}---------$res-------------")
        } catch {
          case ex: Exception => {
            error_count.incrementAndGet()
            ex.printStackTrace()
          }
        } finally {
          latch.countDown()
        }
      })
    })
    latch.await()
    if (error_count.get() != 0) {
      println("There are failed jobs !!!")
      sys.exit(-998)
    }
  }

  private def asyncWatch[T](name: String, f: () => Unit): Unit = {
    val t = new Thread(new Runnable {
      override def run(): Unit = {
        watch(name, f)
      }
    })
    t.setName(name)
    t.start()
  }

  private def watch[T](name: String, fun: () => T): T = {
    val stopWatch = new StopWatch
    stopWatch.start()
    println(
      s"""
         |
         |----------------------- submit [action=$name] begin... -----------------------
         |""".stripMargin)
    logInfo(
      s"""
         |
         |----------------------- submit [action=$name] begin... -----------------------
       """.stripMargin
    )

    val r = fun()
    stopWatch.stop()
    val totalSeconds = stopWatch.getTime(TimeUnit.SECONDS)

    println(
      s"""
         |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
         |
         |""".stripMargin)
    logInfo(
      s"""
         |-------------- submit [action=$name] end, cost ${toTimeStr(totalSeconds)} -------------
         |
       """.stripMargin
    )
    r
  }

  private def toTimeStr(totalSeconds: Long): String = {
    val hours = TimeUnit.SECONDS.toHours(totalSeconds)
    val minutes = TimeUnit.SECONDS.toMinutes(totalSeconds - hours * 3600)
    val seconds = totalSeconds - hours * 3600 - minutes * 60
    val timeStr = if (hours > 0) {
      s"${hours}h${minutes}m${seconds}s"
    } else if (minutes > 0) {
      s"${minutes}m${seconds}s"
    } else {
      s"${seconds}s"
    }
    timeStr
  }
}
