package com.winhc.bigdata.spark.utils

import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging
import org.apache.spark.sql.types.{DataType, StructField, StructType}
import org.apache.spark.sql.{DataFrame, SparkSession}

import scala.annotation.meta.getter
import scala.collection.immutable.ListMap

/**
 * π
 */
trait LoggingUtils extends Logging {
  protected var sqlNo = 1

  @(transient@getter) protected val spark: SparkSession

  def sql(sqlString: String): DataFrame = {
    logInfo(
      s"""
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |Job[${this.getClass.getSimpleName}].SQL[No$sqlNo.]
         |
         |$sqlString
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
       """.stripMargin
    )
    println(
      s"""
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |Job[${this.getClass.getSimpleName}].SQL[No$sqlNo.]
         |
         |$sqlString
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |""".stripMargin)
    sqlNo += 1
    spark.sql(sqlString)
  }

  def showString(dataset: DataFrame, _numRows: Int = 20, truncate: Int = -1): String = {
    val numRows = _numRows.max(0)
    val takeResult = dataset.take(numRows + 1)
    val hasMoreData = takeResult.length > numRows
    val data = takeResult.take(numRows)

    // For array values, replace Seq and Array with square brackets
    // For cells that are beyond `truncate` characters, replace it with the
    // first `truncate-3` and "..."
    val rows: Seq[Seq[String]] = dataset.schema.fieldNames.toSeq +: data.map { row =>
      row.toSeq.map { cell =>
        val str = cell match {
          case null => "null"
          case binary: Array[Byte] => binary.map("%02X".format(_)).mkString("[", " ", "]")
          case array: Array[_] => array.mkString("[", ", ", "]")
          case seq: Seq[_] => seq.mkString("[", ", ", "]")
          case _ => cell.toString
        }
        if (truncate > 0 && str.length > truncate) {
          // do not show ellipses for strings shorter than 4 characters.
          if (truncate < 4) str.substring(0, truncate)
          else str.substring(0, truncate - 3) + "..."
        } else {
          str
        }
      }: Seq[String]
    }

    val sb = new StringBuilder
    val numCols = dataset.schema.fieldNames.length

    // Initialise the width of each column to a minimum value of '3'
    val colWidths = Array.fill(numCols)(3)

    // Compute the width of each column
    for (row <- rows) {
      for ((cell, i) <- row.zipWithIndex) {
        colWidths(i) = math.max(colWidths(i), cell.length)
      }
    }

    // Create SeparateLine
    val sep: String = colWidths.map("-" * _).addString(sb, "+", "+", "+\n").toString()

    // column names
    rows.head.zipWithIndex.map { case (cell, i) =>
      if (truncate > 0) {
        StringUtils.leftPad(cell, colWidths(i))
      } else {
        StringUtils.rightPad(cell, colWidths(i))
      }
    }.addString(sb, "|", "|", "|\n")

    sb.append(sep)

    // data
    rows.tail.map {
      _.zipWithIndex.map { case (cell, i) =>
        if (truncate > 0) {
          StringUtils.leftPad(cell.toString, colWidths(i))
        } else {
          StringUtils.rightPad(cell.toString, colWidths(i))
        }
      }.addString(sb, "|", "|", "|\n")
    }

    sb.append(sep)

    // For Data that has more than "numRows" records
    if (hasMoreData) {
      val rowsString = if (numRows == 1) "row" else "rows"
      sb.append(s"only showing top $numRows $rowsString\n")
    }

    sb.toString()
  }


  def dropAllPartitions(tableName: String): Unit = {
    val array = sql(s"show partitions $tableName").collect().map(_.getString(0))
    val tns = array.map(r => {
      r.split("/").map(t => {
        val record = t.split("=")
        s"${record(0)}='${record(1)}'"
      }).mkString(",")
    })
    logInfo(
      s"""
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |Job[${this.getClass.getSimpleName}].SQL[No$sqlNo.]
         |
         |dropAllPartitions $tableName
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
       """.stripMargin
    )
    println(
      s"""
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |Job[${this.getClass.getSimpleName}].SQL[No$sqlNo.]
         |
         |dropAllPartitions $tableName
         |- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         |""".stripMargin)

    for (elem <- tns) {
      spark.sql(s"ALTER TABLE $tableName DROP IF EXISTS PARTITION($elem)")
    }
  }

  def getPartitions(t: String, expression: String = null): Seq[String] = {
    var list = sql(s"show partitions $t").collect().map(_.getString(0))
    if (expression != null) {
      list = list.filter(s => s.split("/").contains(expression))
    }
    list.map(r => {
      if (r.contains("/")) {
        r.split("/").find(s => s.contains("ds")).map(s => s.split("=")(1)).orNull
      } else {
        r.split("=")(1)
      }
    })
  }

  /*def getPartitions(t: String): Seq[String] = {
    val sql_s = s"show partitions " + t
    sql(sql_s).collect.toList.map(r => r.getString(0)).map(r => {
      if (r.contains("/")) {
        r.split("/").find(s => s.contains("ds")).map(s => s.split("=")(1)).orNull
      } else {
        r.split("=")(1)
      }
    })
  }*/

  def getSecondLastPartitionOrElse(t: String, default: String, expression: String = null): String = {
    val ps = getPartitions(t, expression)
    if (ps.length >= 2) {
      ps(ps.length - 2)
    } else {
      default
    }
  }

  def getLastPartitionsOrElse(t: String, default: String, expression: String = null): String = {
    val ps = getPartitions(t, expression)
    if (ps.nonEmpty) {
      ps.last
    } else {
      default
    }
  }

  def getHeadPartitionsOrElse(t: String, default: String, expression: String = null): String = {
    val ps = getPartitions(t, expression)
    if (ps.nonEmpty) {
      ps.head
    } else {
      default
    }
  }

  def getColumns(t: String): Seq[String] = {
    spark.table(t).columns.seq
  }

  def getSchema(map: ListMap[String, DataType]): StructType = {
    StructType(map.map(e => StructField(e._1, e._2)).toArray)
  }

  def getEnvProjectName(env: String, projectName: String): String = {
    if (env.equals("dev")) {
      projectName
    } else {
      projectName.substring(0, projectName.length - 4)
    }
  }


  def generateAllTabSql(tableName: String, project: String, maxDs: String = null): (String, Seq[String], String) = {
    val inc_ads_tab = s"$project.inc_ads_$tableName"
    val ads_tab = s"$project.ads_$tableName"
    val cols = getColumns(inc_ads_tab).intersect(getColumns(ads_tab))

    val ads_last_ds = getLastPartitionsOrElse(ads_tab, null)
    if (ads_last_ds == null) {
      throw new RuntimeException(s"$ads_tab ds is null !")
    }
    val p_b = if (cols.contains("rowkey")) "rowkey" else if (cols.contains("company_id")) "company_id" else throw new RuntimeException(s"$ads_tab partition key is null !")

    val max = if(maxDs==null) "" else s" and ds <= $maxDs"

    (
      s"""
         |SELECT  ${cols.mkString(",")}
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY $p_b ORDER BY ds DESC ) AS xjk_num
         |            FROM    (
         |                        SELECT  ${cols.mkString(",")}
         |                        FROM    $ads_tab
         |                        WHERE   ds = $ads_last_ds
         |                        UNION ALL
         |                        SELECT  ${cols.mkString(",")}
         |                        FROM    $inc_ads_tab
         |                        WHERE   ds > $ads_last_ds $max
         |                    ) AS all_t1
         |        ) AS all_t2
         |WHERE   all_t2.xjk_num = 1
         |""".stripMargin, cols, p_b)
  }
}
